@@ -82,7 +82,7 @@ void QFitsMultiBuffer::init(dpuserType *dpt, int dbg, QString dbg_upper) {
     }
     grid = new QGridLayout(gridWidget);
     grid->setSpacing(3);
-    grid->setMargin(3);
+    grid->setContentsMargins(3, 3, 3, 3);
 
     int listSize    = dpData->dparrvalue->size(),
         nr_columns  = ceil(sqrt(listSize)),
@@ -512,8 +512,8 @@ void QFitsMultiBuffer::zoomTextChanged(double zoomValue, FitZoom fz) {
 
             int nrCol = grid->columnCount(),
                 nrRow = grid->rowCount(),
-                w = (gridWidget->width()  - 2*grid->margin() - (nrCol-1)*grid->spacing()) / nrCol,
-                h = (gridWidget->height() - wedgeSpectrumCorrection - 2*grid->margin() - (nrRow-1)*grid->spacing()) / nrRow;
+                w = (gridWidget->width()  - grid->contentsMargins().right() - grid->contentsMargins().left() - (nrCol-1)*grid->spacing()) / nrCol,
+                h = (gridWidget->height() - wedgeSpectrumCorrection - grid->contentsMargins().bottom() - grid->contentsMargins().top() - (nrRow-1)*grid->spacing()) / nrRow;
 
             zoomValue = DBL_MAX;
             if (cubeSpecHomogenSB) {
@@ -936,5 +936,7 @@ void QFitsMultiBuffer::copyImage(int mode) {
         for (int i = 0; i < bufferList.size(); i++) {
             bufferList.at(i)->copyImage(mode);
         }
+    } else {
+        getTableWidget()->copyImage(mode);
     }
 }
