@@ -995,16 +995,7 @@ void QFitsSingleBuffer::setSpecChannelMinX(int channelMinX) {
         double  crval       = f->getCRVAL(dim),
                 crpix       = f->getCRPIX(dim),
                 cdelt       = f->getCDELT(dim),
-                physMin     = crval + (channelMinX - crpix) * cdelt;
-
-        if ((crpix-1.0 < 1e-8) && (crpix-1.0 > -1e-8) &&
-            (crval-1.0 < 1e-8) && (crval-1.0 > -1e-8) &&
-            (cdelt-1.0 < 1e-8) && (cdelt-1.0 > -1e-8))
-        {
-            bufAppearance.enableToolbarPhysicalRange = false;
-        } else {
-            bufAppearance.enableToolbarPhysicalRange = true;
-        }
+                physMin     = pixwave(channelMinX, crpix, crval, cdelt, f->isAxisLog(dim)); //crval + (channelMinX - crpix) * cdelt;
 
         specChannelMinX = channelMinX;
         specPhysMinX = physMin;
@@ -1066,16 +1057,7 @@ void QFitsSingleBuffer::setSpecChannelMaxX(int channelMaxX) {
         double  crval       = f->getCRVAL(dim),
                 crpix       = f->getCRPIX(dim),
                 cdelt       = f->getCDELT(dim),
-                physMax     = crval + (channelMaxX - crpix) * cdelt;
-
-        if ((crpix-1.0 < 1e-8) && (crpix-1.0 > -1e-8) &&
-            (crval-1.0 < 1e-8) && (crval-1.0 > -1e-8) &&
-            (cdelt-1.0 < 1e-8) && (cdelt-1.0 > -1e-8))
-        {
-            bufAppearance.enableToolbarPhysicalRange = false;
-        } else {
-            bufAppearance.enableToolbarPhysicalRange = true;
-        }
+                physMax     = pixwave(channelMaxX, crpix, crval, cdelt, f->isAxisLog(dim)); //crval + (channelMaxX - crpix) * cdelt;
 
         specChannelMaxX = channelMaxX;
         specPhysMaxX = physMax;
@@ -1124,15 +1106,8 @@ void QFitsSingleBuffer::setSpecPhysMinX(double physMinX) {
             double  crval       = f->getCRVAL(QFV::Horizontal),
                     crpix       = f->getCRPIX(QFV::Horizontal),
                     cdelt       = f->getCDELT(QFV::Horizontal);
-            int     channelMin  = (int)(crpix + (physMinX - crval) / cdelt + 0.5);
-            if ((crpix-1.0 < 1e-8) && (crpix-1.0 > -1e-8) &&
-                (crval-1.0 < 1e-8) && (crval-1.0 > -1e-8) &&
-                (cdelt-1.0 < 1e-8) && (cdelt-1.0 > -1e-8))
-            {
-                bufAppearance.enableToolbarPhysicalRange = false;
-            } else {
-                bufAppearance.enableToolbarPhysicalRange = true;
-            }
+//            int     channelMin  = (int)(crpix + (physMinX - crval) / cdelt + 0.5);
+            int     channelMin  = wavepix(physMinX, crpix, crval, cdelt, f->isAxisLog(QFV::Horizontal));
 
             specPhysMinX    = physMinX;
             specChannelMinX = channelMin;
@@ -1177,15 +1152,8 @@ void QFitsSingleBuffer::setSpecPhysMaxX(double physMaxX) {
             double  crval       = f->getCRVAL(QFV::Horizontal),
                     crpix       = f->getCRPIX(QFV::Horizontal),
                     cdelt       = f->getCDELT(QFV::Horizontal);
-            int     channelMax  = (int)(crpix + (physMaxX - crval) / cdelt + 0.5);
-            if ((crpix-1.0 < 1e-8) && (crpix-1.0 > -1e-8) &&
-                (crval-1.0 < 1e-8) && (crval-1.0 > -1e-8) &&
-                (cdelt-1.0 < 1e-8) && (cdelt-1.0 > -1e-8))
-            {
-                bufAppearance.enableToolbarPhysicalRange = false;
-            } else {
-                bufAppearance.enableToolbarPhysicalRange = true;
-            }
+//            int     channelMax  = (int)(crpix + (physMaxX - crval) / cdelt + 0.5);
+            int     channelMax  = wavepix(physMaxX, crpix, crval, cdelt, f->isAxisLog(QFV::Horizontal));
 
             specPhysMaxX    = physMaxX;
             specChannelMaxX = channelMax;
@@ -1488,21 +1456,26 @@ void QFitsSingleBuffer::updateLinemapDialog() {
             }
 
             QString text1, text2, textInfo;
+            dpString fstring;
             if (dpData->fvalue->getCDELT(3) != 0.0) {
                 double start_wavelength = dpData->fvalue->getCRVAL(3) - dpData->fvalue->getCDELT(3) * (dpData->fvalue->getCRPIX(3) - 1.0);
                 double end_wavelength = start_wavelength + dpData->fvalue->getCDELT(3) * Naxis(3);
                 double width_wavelength = dpData->fvalue->getCDELT(3) * dlg->getLinemapWidthValue();
 
-                text1.sprintf("%8.5f", start_wavelength);
-                text2.sprintf("%8.5f", end_wavelength);
+                fstring.sprintf("%8.5f", start_wavelength);
+                text1 = fstring.c_str();
+                fstring.sprintf("%8.5f", end_wavelength);
+                text2 = fstring.c_str();
                 char value[256];
                 dpData->fvalue->GetStringKey("CUNIT3", value);
-                textInfo.sprintf("Center at %8.5f +- %8.5f %s", start_wavelength,width_wavelength, value);
+                fstring.sprintf("Center at %8.5f +- %8.5f %s", start_wavelength,width_wavelength, value);
+                textInfo = fstring.c_str();
                 dlg->setLinemapWavelengthSliderInformation(&text1, &textInfo, &text2);
             } else {
                 text1 = "1";
                 textInfo = "No wavelength information available";
-                text2.sprintf("%i", Naxis(3));
+                fstring.sprintf("%i", Naxis(3));
+                text2 = fstring.c_str();
                 dlg->setLinemapWavelengthSliderInformation(&text1, &textInfo, &text2);
             }
         }
@@ -1647,6 +1620,7 @@ void QFitsSingleBuffer::setData() {
                 copy->copy(*(dpData->fvalue));
                 setCubeDisplay1D(copy);
                 widget1D->setData();
+
                 break;
             }
         }
