@@ -14,10 +14,10 @@
 #include "QFitsMultiBuffer.h"
 #include "QFitsWidgetTable.h"
 #include "QFitsView1D.h"
+#include "QFitsWidget1D.h"
 #include "QFitsMarkers.h"
 #include "QFitsGlobal.h"
 #include "qtdpuser.h"
-#include "resources/xicon.xpm"
 
 //------------------------------------------------------------------------------
 //         QFitsCubeSpectrum
@@ -35,28 +35,35 @@ QFitsCubeSpectrum::QFitsCubeSpectrum(QFitsMainWindow *parent)
     ignoreValue     = -1e10;
 
     QPalette pal(palette());
-    pal.setColor(QPalette::Background, Qt::Window);
+//    pal.setColor(QPalette::Background, Qt::Window);
     setPalette(pal);
 
     setWindowTitle("QFitsView - Spectrum Plot");
     plotter = new QFitsCubeSpectrumViewer(this);
 
-    minlabel = new QLabel("Minimum:", this);
-    minlabel->setGeometry(0, 0, 70, 20);
+    minlabel = new QLabel("Minimum: ", this);
+    minlabel->adjustSize();
+//    minlabel->setGeometry(0, 0, 70, 20);
 
-    minEdit = new QLineEdit("0", this);
-    minEdit->setGeometry(70, 0, 50, 20);
-    minEdit->setValidator(new QDoubleValidator(minEdit));
+    minEdit = new dpDoubleEdit("0", this);
+    minEdit->adjustSize();
+    minEdit->move(minlabel->width(), 0);
+//    minEdit->setGeometry(70, 0, 50, 20);
+//    minEdit->setValidator(new QDoubleValidator(minEdit));
     minEdit->setEnabled(false);
     connect(minEdit, SIGNAL(textChanged(const QString &)),
             this, SLOT(setMin(const QString &)));
 
-    maxlabel = new QLabel("Maximum:", this);
-    maxlabel->setGeometry(120, 0, 70, 20);
+    maxlabel = new QLabel(" Maximum: ", this);
+    maxlabel->adjustSize();
+//    maxlabel->move(minEdit->x() + minEdit->width(), 0);
+//    maxlabel->setGeometry(120, 0, 70, 20);
 
-    maxEdit = new QLineEdit("100", this);
-    maxEdit->setGeometry(190, 0, 50, 20);
-    maxEdit->setValidator(new QDoubleValidator(maxEdit));
+    maxEdit = new dpDoubleEdit("100", this);
+    maxEdit->adjustSize();
+//    maxEdit->move(maxlabel->x() + maxlabel->width(), 0);
+//    maxEdit->setGeometry(190, 0, 50, 20);
+//    maxEdit->setValidator(new QDoubleValidator(maxEdit));
     maxEdit->setEnabled(false);
     connect(maxEdit, SIGNAL(textChanged(const QString &)),
             this, SLOT(setMax(const QString &)));
@@ -65,8 +72,10 @@ QFitsCubeSpectrum::QFitsCubeSpectrum(QFitsMainWindow *parent)
     connect(rangeControl, SIGNAL(channelRangeChanged()),
             this, SLOT(update()));
 
-    autoScaleButton = new QCheckBox("Auto Scale", this);
-    autoScaleButton->setGeometry(250, 0, 100, 20);
+    autoScaleButton = new QCheckBox("Auto Scale ", this);
+    autoScaleButton->adjustSize();
+//    autoScaleButton->move(maxEdit->x() + maxEdit->width() + fontMetrics().horizontalAdvance('M'), 0);
+//    autoScaleButton->setGeometry(250, 0, 100, 20);
     autoScaleButton->setChecked(true);
     connect(autoScaleButton, SIGNAL(toggled(bool)),
             minEdit, SLOT(setDisabled(bool)));
@@ -76,43 +85,57 @@ QFitsCubeSpectrum::QFitsCubeSpectrum(QFitsMainWindow *parent)
             plotter, SLOT(setAutoScale(bool)));
 
     methodButton = new QComboBox(this);
-    methodButton->setGeometry(350, 0, 145, 20);
+//    methodButton->setGeometry(350, 0, 145, 20);
     methodButton->insertItem(0, "Single Pixel");
     methodButton->insertItem(1, "Circular");
     methodButton->insertItem(2, "Circular-Annular");
+    methodButton->adjustSize();
+//    methodButton->move(autoScaleButton->x() + autoScaleButton->width(), 0);
     connect(methodButton, SIGNAL(activated(int)),
             this, SLOT(setMethod(int)));
 
-    r1label = new QLabel("R1:", this);
-    r1label->setGeometry(505, 0, 20, 20);
+    r1label = new QLabel(" R1: ", this);
+    r1label->adjustSize();
+//    r1label->move(methodButton->x() + methodButton->width(), 0);
+//    r1label->setGeometry(505, 0, 20, 20);
     r1label->setEnabled(false);
 
     r1edit = new QSpinBox(this);
     r1edit->setMinimum(0);
     r1edit->setMaximum(1000);
+    r1edit->adjustSize();
+    r1edit->resize(r1edit->width(), minEdit->height());
     r1edit->setSingleStep(1);
     r1edit->setValue(1);
-    r1edit->setGeometry(525, 0, 50, 20);
+//    r1edit->move(r1label->x() + r1label->width(), 0);
+//    r1edit->setGeometry(525, 0, 50, 20);
     r1edit->setEnabled(false);
     connect(r1edit, SIGNAL(valueChanged(int)),
             this, SLOT(setR1(int)));
 
-    r2label = new QLabel("R2:", this);
-    r2label->setGeometry(585, 0, 20, 20);
+    r2label = new QLabel(" R2: ", this);
+    r2label->adjustSize();
+//    r2label->move(r1edit->x() + r1edit->width(), 0);
+//    r2label->setGeometry(585, 0, 20, 20);
     r2label->setEnabled(false);
 
     r2edit = new QSpinBox(this);
     r2edit->setMinimum(0);
     r2edit->setMaximum(1000);
+    r2edit->adjustSize();
+    r2edit->resize(r2edit->width(), minEdit->height());
     r2edit->setSingleStep(1);
     r2edit->setValue(2);
-    r2edit->setGeometry(605, 0, 50, 20);
+//    r2edit->move(r2label->x() + r2label->width(), 0);
+//    r2edit->setGeometry(605, 0, 50, 20);
     r2edit->setEnabled(false);
     connect(r2edit, SIGNAL(valueChanged(int)),
             this, SLOT(setR2(int)));
 
     useStandardStar = new QCheckBox("Divide Std", this);
-    useStandardStar->setGeometry(658, 0, 100, 20);
+    useStandardStar->adjustSize();
+//    useStandardStar->move(r2edit->x() + r2edit->width(), 0);
+//    useStandardStar->setGeometry(658, 0, 100, 20);
     useStandardStar->setChecked(false);
     useStandardStar->setEnabled(false);
     connect(useStandardStar, SIGNAL(clicked()),
@@ -123,11 +146,40 @@ QFitsCubeSpectrum::QFitsCubeSpectrum(QFitsMainWindow *parent)
     statisticsButton->insertItem(1, "Average");
     statisticsButton->insertItem(2, "Median");
     statisticsButton->adjustSize();
-    statisticsButton->move(useStandardStar->x() + useStandardStar->width() + 10, 0);
+//    statisticsButton->move(useStandardStar->x() + useStandardStar->width() + fontMetrics().horizontalAdvance('M'), 0);
     connect(statisticsButton, SIGNAL(activated(int)),
             this, SLOT(setStatistics(int)));
 
-    hideButton = new QFitsSimplestButton(QPixmap(xicon), this);
+    QWidget *topWidget = new QWidget(this);
+    QHBoxLayout *toplayout = new QHBoxLayout(topWidget);
+    toplayout->setContentsMargins(0, 0, 0, 0);
+//    toplayout->setSpacing(0);
+    toplayout->addWidget(minlabel);
+    toplayout->addWidget(minEdit);
+    toplayout->addWidget(maxlabel);
+    toplayout->addWidget(maxEdit);
+    toplayout->addWidget(rangeControl);
+    toplayout->addWidget(autoScaleButton);
+    toplayout->addWidget(methodButton);
+    toplayout->addWidget(r1label);
+    toplayout->addWidget(r1edit);
+    toplayout->addWidget(r2label);
+    toplayout->addWidget(r2edit);
+    toplayout->addWidget(useStandardStar);
+    toplayout->addWidget(statisticsButton);
+    topWidget->adjustSize();
+    topWidget->setFixedSize(topWidget->width(), autoScaleButton->height());
+
+    rangeControl->setFixedHeight(rangeControl->height());
+
+    QVBoxLayout *vlayout = new QVBoxLayout(this);
+    vlayout->setSpacing(0);
+    vlayout->setContentsMargins(0, 0, 0, 0);
+    vlayout->addWidget(topWidget);
+    vlayout->addWidget(plotter);
+    vlayout->addWidget(rangeControl);
+
+    hideButton = new QFitsSimplestButton(QIcon(":/xicon.svg").pixmap(10, 10), this);
     hideButton->setGeometry(width() - 12, 0, 12, 12);
     hideButton->setToolTip("Close");
     connect(hideButton, SIGNAL(clicked()),
@@ -230,9 +282,9 @@ void QFitsCubeSpectrum::paintEvent(QPaintEvent *) {
 }
 
 void QFitsCubeSpectrum::resizeEvent(QResizeEvent *e) {
-    plotter->setGeometry(0, 20, e->size().width(), e->size().height() - 20-rangeControl->height());
-    rangeControl->setGeometry(0, e->size().height()-rangeControl->height(),
-                              e->size().width(), rangeControl->height());
+//    plotter->setGeometry(0, 20, e->size().width(), e->size().height() - 20-rangeControl->height());
+//    rangeControl->setGeometry(0, e->size().height()-rangeControl->height(),
+//                              e->size().width(), rangeControl->height());
     hideButton->setGeometry(e->size().width() - 12, 0, 12, 12);
 }
 
@@ -261,7 +313,7 @@ void QFitsCubeSpectrum::enableControls(bool b) {
 }
 
 void QFitsCubeSpectrum::resetRangeControl() {
-    rangeControl->setSpectrum(0, 0, 0, 0);
+    rangeControl->setSpectrum(0, 0, 0, 0, false);
 }
 
 void QFitsCubeSpectrum::changeWorkArray() {
@@ -278,6 +330,7 @@ void QFitsCubeSpectrum::changeWorkArray() {
         double  crpix       = 0.,
                 crval       = 0.,
                 cdelt       = 0.;
+        bool    logwave     = false;
 
         dpt = sb->getDpData();
         if (dpt->type == typeFits) {
@@ -290,12 +343,14 @@ void QFitsCubeSpectrum::changeWorkArray() {
                     crpix       = f->getCRPIX(3),
                     crval       = f->getCRVAL(3),
                     cdelt       = f->getCDELT(3);
+                    logwave     = f->isAxisLog(3);
                 } else if (f->Naxis(0) == 2) {
                     int dim = sb->getCubeSpecOrientation();
                     nelements   = f->Naxis(dim);
                     crpix       = f->getCRPIX(dim),
                     crval       = f->getCRVAL(dim),
                     cdelt       = f->getCDELT(dim);
+                    logwave     = f->isAxisLog(dim);
                 } else if ((f->extensionType == BINTABLE) || (f->extensionType == TABLE) || (f->Naxis(0) == 1)) {
                     Fits *display1D = sb->getCubeDisplay1D();
                     if (display1D != NULL) {
@@ -303,11 +358,12 @@ void QFitsCubeSpectrum::changeWorkArray() {
                         crpix       = f->getCRPIX(1),
                         crval       = f->getCRVAL(1),
                         cdelt       = f->getCDELT(1);
+                        logwave     = f->isAxisLog(3);
                     }
                 }
             }
 
-            rangeControl->setSpectrum(nelements, crpix, crval, cdelt);
+            rangeControl->setSpectrum(nelements, crpix, crval, cdelt, logwave);
 
             if (sb->showSpectrum()) {
                 show();
@@ -335,12 +391,12 @@ void QFitsCubeSpectrum::changeWorkArrayTable() {
 
         Fits *f = sb->getCubeDisplay1D();
         if (f != NULL) {
-            rangeControl->setSpectrum(f->Naxis(1), f->getCRPIX(1), f->getCRVAL(1), f->getCDELT(1));
+            rangeControl->setSpectrum(f->Naxis(1), f->getCRPIX(1), f->getCRVAL(1), f->getCDELT(1), false);
         } else {
-            rangeControl->setSpectrum(0., 0., 0., 0.);
+            rangeControl->setSpectrum(0., 0., 0., 0., false);
         }
     } else {
-        rangeControl->setSpectrum(0., 0., 0., 0.);
+        rangeControl->setSpectrum(0., 0., 0., 0., false);
     }
     fitsMainWindow->spectrum->update();
 }
@@ -406,7 +462,7 @@ void QFitsCubeSpectrum::saveAscii(QFitsSingleBuffer *sb) {
             }
 
             for (int i = 0; i < line.Nelements(); i++) {
-                if (fprintf(fd, "%20.10f %20.10f\n", start + i * delta, line.ValueAt(i)) < 0) {
+                if (fprintf(fd, "%20.10f %20.10f\n", pixwave(i, f->getCRPIX(3), f->getCRVAL(3), f->getCDELT(3), f->isAxisLog(3)), line.ValueAt(i)) < 0) {
                     QMessageBox::warning(this, "QFitsView", "Could not write data");
                     fclose(fd);
                     return;
@@ -490,7 +546,7 @@ void QFitsCubeSpectrum::saveFitsSpectrum(QFitsSingleBuffer *sb) {
     }
 }
 
-void QFitsCubeSpectrum::newBufferSpectrum(QFitsSingleBuffer *sb) {;
+void QFitsCubeSpectrum::newBufferSpectrum(QFitsSingleBuffer *sb) {
     if ((sb != NULL) &&
         (sb->getDpData()->type == typeFits) &&
         (sb->getDpData()->fvalue != NULL))
@@ -742,13 +798,12 @@ void QFitsCubeSpectrum::setupMarkers() {
 QFitsSpectrumRangeControl::QFitsSpectrumRangeControl(QWidget *parent)
                                             : QWidget(parent)
 {
-    setFixedHeight(20);
+//    setFixedHeight(20);
 
     channelMin = new QSpinBox(this);
     channelMin->setMinimum(1);
     channelMin->setMaximum(1e100);
     channelMin->setSingleStep(1);
-    channelMin->adjustSize();
     channelMin->setToolTip("x-Axis minimum in channels");
     channelMin->setKeyboardTracking(false);
     connect(channelMin, SIGNAL(valueChanged(int)),
@@ -758,7 +813,6 @@ QFitsSpectrumRangeControl::QFitsSpectrumRangeControl(QWidget *parent)
     channelMax->setMinimum(2);
     channelMax->setMaximum(1e100);
     channelMax->setSingleStep(1);
-    channelMax->adjustSize();
     channelMax->setToolTip("x-Axis maximum in channels");
     channelMax->setKeyboardTracking(false);
     connect(channelMax, SIGNAL(valueChanged(int)),
@@ -784,6 +838,9 @@ QFitsSpectrumRangeControl::QFitsSpectrumRangeControl(QWidget *parent)
     connect(physicalMax, SIGNAL(valueChanged(double)),
             this, SLOT(setPhysicalMax(double)));
 
+    channelMin->resize(physicalMin->size());
+    channelMax->resize(physicalMax->size());
+
     rangeScroller = new QScrollBar(Qt::Horizontal, this);
     rangeScroller->setMinimum(1);
     rangeScroller->setTracking(true);
@@ -798,7 +855,8 @@ QFitsSpectrumRangeControl::QFitsSpectrumRangeControl(QWidget *parent)
 void QFitsSpectrumRangeControl::setSpectrum(int   t_nelements,
                                            double t_crpix,
                                            double t_crval,
-                                           double t_cdelt)
+                                           double t_cdelt,
+                                           bool t_logwave)
 {
     updateAll = false;
 
@@ -806,6 +864,7 @@ void QFitsSpectrumRangeControl::setSpectrum(int   t_nelements,
     crpix       = t_crpix;
     crval       = t_crval;
     cdelt       = t_cdelt;
+    logwave     = t_logwave;
 
     if (nelements == 0) {
         channelMin->setMinimum(0);
@@ -844,11 +903,15 @@ void QFitsSpectrumRangeControl::setSpectrum(int   t_nelements,
             channelMax->setMinimum(2);
             channelMax->setMaximum(nelements);
 
-            min = crval + (1 - crpix) * cdelt;
-            max = crval + (nelements - crpix) * cdelt;
+//            min = crval + (1 - crpix) * cdelt;
+//            max = crval + (nelements - crpix) * cdelt;
+            min = pixwave(1, crpix, crval, cdelt, logwave);
+            max = pixwave(nelements, crpix, crval, cdelt, logwave);
             physicalMin->setMinimum(min);
-            physicalMin->setMaximum(crval + ((nelements - 1) - crpix) * cdelt);
-            physicalMax->setMinimum(crval + (2 - crpix) * cdelt);
+//            physicalMin->setMaximum(crval + ((nelements - 1) - crpix) * cdelt);
+            physicalMin->setMaximum(pixwave(nelements - 1, crpix, crval, cdelt, logwave));
+//            physicalMax->setMinimum(crval + (2 - crpix) * cdelt);
+            physicalMax->setMinimum(pixwave(2, crpix, crval, cdelt, logwave));
             physicalMax->setMaximum(max);
 
             rangeScroller->setInvertedAppearance(false);
@@ -858,11 +921,15 @@ void QFitsSpectrumRangeControl::setSpectrum(int   t_nelements,
             channelMax->setMinimum(1);
             channelMax->setMaximum(nelements - 1);
 
-            min = crval + (nelements - crpix) * cdelt;
-            max = crval + (1 - crpix) * cdelt;
+//            min = crval + (nelements - crpix) * cdelt;
+//            max = crval + (1 - crpix) * cdelt;
+            min = pixwave(nelements, crpix, crval, cdelt, logwave);
+            max = pixwave(1, crpix, crval, cdelt, logwave);
             physicalMin->setMinimum(min);
-            physicalMin->setMaximum(crval + (2 - crpix) * cdelt);
-            physicalMax->setMinimum(crval + ((nelements - 1) - crpix) * cdelt);
+//            physicalMin->setMaximum(crval + (2 - crpix) * cdelt);
+            physicalMin->setMaximum(pixwave(2, crpix, crval, cdelt, logwave));
+//            physicalMax->setMinimum(crval + ((nelements - 1) - crpix) * cdelt);
+            physicalMax->setMinimum(pixwave(nelements - 1, crpix, crval, cdelt, logwave));
             physicalMax->setMaximum(max);
 
             rangeScroller->setInvertedAppearance(true);
@@ -871,7 +938,16 @@ void QFitsSpectrumRangeControl::setSpectrum(int   t_nelements,
         physicalMin->setSingleStep(fabs(cdelt));
         physicalMax->setSingleStep(fabs(cdelt));
 
-        QFitsSingleBuffer *sb = fitsMainWindow->getActualSB();
+        QFitsWidget1D *oneD = dynamic_cast<QFitsWidget1D *>(parent());
+        QFitsSingleBuffer *sb;
+        if (oneD != NULL) {
+            sb = oneD->getMyBuffer();
+        } else {
+            sb = fitsMainWindow->getActualSB();
+        }
+
+//        QFitsSingleBuffer *sb = fitsMainWindow->getActualSB();
+
         if ((sb != NULL) &&
             (sb->getDpData()->fvalue != NULL) &&
             (sb->getDpData()->fvalue->extensionType != EMPTY))
@@ -885,8 +961,8 @@ void QFitsSpectrumRangeControl::setSpectrum(int   t_nelements,
         }
     }
 
-    channelMin->adjustSize();
-    channelMax->adjustSize();
+//    channelMin->adjustSize();
+//    channelMax->adjustSize();
     physicalMin->adjustSize();
     physicalMax->adjustSize();
     resizeControls(width(), height());
@@ -899,9 +975,15 @@ void QFitsSpectrumRangeControl::setSpectrum(int   t_nelements,
 }
 
 void QFitsSpectrumRangeControl::setPhysicalRange(double min, double max) {
-    QFitsBaseBuffer *bb = fitsMainWindow->getActualSB();
-    if (bb == NULL) {
-        bb = dynamic_cast<QFitsMultiBuffer*>(fitsMainWindow->getCurrentBuffer());
+    QFitsWidget1D *oneD = dynamic_cast<QFitsWidget1D *>(parent());
+    QFitsBaseBuffer *bb;
+    if (oneD != NULL) {
+        bb = oneD->getMyBuffer();
+    } else {
+        bb = fitsMainWindow->getActualSB();
+        if (bb == NULL) {
+            bb = dynamic_cast<QFitsMultiBuffer*>(fitsMainWindow->getCurrentBuffer());
+        }
     }
 
     if (bb != NULL) {
@@ -913,11 +995,11 @@ void QFitsSpectrumRangeControl::setPhysicalRange(double min, double max) {
                 tempr   = 0.;
 
         if (cdelt >= 0) {
-            if (physMin < crval + (1 - crpix) * cdelt) {
-                physMin = crval + (1 - crpix) * cdelt;
+            if (physMin < pixwave(1, crpix, crval, cdelt, logwave)) {
+                physMin = pixwave(1, crpix, crval, cdelt, logwave);
             }
-            if (physMax > crval + (nelements - crpix) * cdelt) {
-                physMax = crval + (nelements - crpix) * cdelt;
+            if (physMax > pixwave(nelements, crpix, crval, cdelt, logwave)) {
+                physMax = pixwave(nelements, crpix, crval, cdelt, logwave);
             }
             if (physMin > physMax) {
                 SWAP(physMin, physMax);
@@ -925,16 +1007,16 @@ void QFitsSpectrumRangeControl::setPhysicalRange(double min, double max) {
             if (fabs(physMin - physMax) < fabs(cdelt) / 10.) {
                 physMax = physMin + cdelt;
             }
-            if (physMax > crval + (nelements - crpix) * cdelt) {
-                physMin = crval + (nelements - 1 - crpix) * cdelt;
-                physMax = crval + (nelements - crpix) * cdelt;
+            if (physMax > pixwave(nelements, crpix, crval, cdelt, logwave)) {
+                physMin = pixwave(nelements - 1, crpix, crval, cdelt, logwave);
+                physMax = pixwave(nelements, crpix, crval, cdelt, logwave);
             }
         } else {
-            if (physMin < crval + (nelements - crpix) * cdelt) {
-                physMin = crval + (nelements - crpix) * cdelt;
+            if (physMin < pixwave(nelements, crpix, crval, cdelt, logwave)) {
+                physMin = pixwave(nelements, crpix, crval, cdelt, logwave);
             }
-            if (physMax > crval + (1 - crpix) * cdelt) {
-                physMax = crval + (1 - crpix) * cdelt;
+            if (physMax > pixwave(1, crpix, crval, cdelt, logwave)) {
+                physMax = pixwave(1, crpix, crval, cdelt, logwave);
             }
             if (physMin > physMax) {
                 SWAP(physMin, physMax);
@@ -942,8 +1024,8 @@ void QFitsSpectrumRangeControl::setPhysicalRange(double min, double max) {
             if (fabs(physMin - physMax) < fabs(cdelt) / 10.) {
                 physMax = physMin - cdelt;
             }
-            if (physMax > crval + (1 - crpix) * cdelt) {
-                physMax = crval + (1 - crpix) * cdelt;
+            if (physMax > pixwave(1, crpix, crval, cdelt, logwave)) {
+                physMax = pixwave(1, crpix, crval, cdelt, logwave);
                 physMin = physMax +cdelt;
             }
         }
@@ -970,10 +1052,17 @@ void QFitsSpectrumRangeControl::setPhysicalRange(double min, double max) {
 }
 
 void QFitsSpectrumRangeControl::setChannelRange(int min, int max) {
-    QFitsBaseBuffer *bb = fitsMainWindow->getActualSB();
-    if (bb == NULL) {
-        bb = dynamic_cast<QFitsMultiBuffer*>(fitsMainWindow->getCurrentBuffer());
+    QFitsWidget1D *oneD = dynamic_cast<QFitsWidget1D *>(parent());
+    QFitsBaseBuffer *bb;
+    if (oneD != NULL) {
+        bb = oneD->getMyBuffer();
+    } else {
+        bb = fitsMainWindow->getActualSB();
+        if (bb == NULL) {
+            bb = dynamic_cast<QFitsMultiBuffer*>(fitsMainWindow->getCurrentBuffer());
+        }
     }
+
     if (bb != NULL) {
         updateAll = false;
 
@@ -1184,8 +1273,8 @@ void QFitsCubeSpectrumViewer::paintEvent(QPaintEvent*) {
         drawAxis(&p, ymin, ymax);
 
         // flip coordinate system in Y for subsequent painting
-        QMatrix origMatrix = p.worldMatrix();
-        p.setWorldMatrix(m);
+        QTransform origMatrix = p.worldTransform();
+        p.setWorldTransform(m);
         p.setClipRect(0, -1, plotwidth, plotheight+1);
 
         // paint continuum spectrum (green filled)
@@ -1212,8 +1301,10 @@ void QFitsCubeSpectrumViewer::paintEvent(QPaintEvent*) {
             switch (sb->getCubeMode()) {
                 case DisplayCubeSingle:
                     // paint narrow (single) source rectangle (light grey filled)
-                    c1 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) - crpix) * cdelt - cdelt / 2.) + 0.5);
-                    c2 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) - crpix) * cdelt + cdelt / 2.) + 0.5);
+//                    c1 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) - crpix) * cdelt - cdelt / 2.) + 0.5);
+//                    c2 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) - crpix) * cdelt + cdelt / 2.) + 0.5);
+                    c1 = wavelengthToPixel(pixwave(sb->getCubeCenter(QFV::Default) - 0.5, crpix, crval, cdelt, logwave));
+                    c2 = wavelengthToPixel(pixwave(sb->getCubeCenter(QFV::Default) + 0.5, crpix, crval, cdelt, logwave));
                     markarray.setPoint(0, c1, 0);
                     markarray.setPoint(1, c1, plotheight);
                     markarray.setPoint(2, c2, plotheight);
@@ -1225,8 +1316,10 @@ void QFitsCubeSpectrumViewer::paintEvent(QPaintEvent*) {
                 case DisplayCubeAverage:
                 case DisplayCubeMedian:
                     // paint source rectangle filling the whole spectrum (light grey filled)
-                    c1 = (int)(wavelengthToPixel(crval + (1 - crpix) * cdelt - cdelt / 2.) + 0.5);
-                    c2 = (int)(wavelengthToPixel(crval + (sb->Naxis(sb->getCubeSpecOrientation()) - crpix) * cdelt + cdelt / 2.) + 0.5);
+//                    c1 = (int)(wavelengthToPixel(crval + (1 - crpix) * cdelt - cdelt / 2.) + 0.5);
+//                    c2 = (int)(wavelengthToPixel(crval + (sb->Naxis(sb->getCubeSpecOrientation()) - crpix) * cdelt + cdelt / 2.) + 0.5);
+                    c1 = wavelengthToPixel(pixwave(0.5, crpix, crval, cdelt, logwave));
+                    c2 = wavelengthToPixel(pixwave(sb->Naxis(sb->getCubeSpecOrientation()) + 0.5, crpix, crval, cdelt, logwave));
                     if (c1 > c2) {
                         int tmp = c1;
                         c1 = c2;
@@ -1247,8 +1340,10 @@ void QFitsCubeSpectrumViewer::paintEvent(QPaintEvent*) {
                     {
                         // in TableView the linewidth is set to 0 when all rows are visible
                         // don't draw the rectangle in this case
-                        c1 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) - sb->getLineWidth(QFV::Default) - crpix) * cdelt - cdelt / 2.) + 0.5);
-                        c2 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineWidth(QFV::Default) - crpix) * cdelt + cdelt / 2.) + 0.5);
+//                        c1 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) - sb->getLineWidth(QFV::Default) - crpix) * cdelt - cdelt / 2.) + 0.5);
+//                        c2 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineWidth(QFV::Default) - crpix) * cdelt + cdelt / 2.) + 0.5);
+                        c1 = wavelengthToPixel(pixwave(sb->getCubeCenter(QFV::Default) - sb->getLineWidth(QFV::Default) - 0.5, crpix, crval, cdelt, logwave));
+                        c2 = wavelengthToPixel(pixwave(sb->getCubeCenter(QFV::Default) + sb->getLineWidth(QFV::Default) + 0.5, crpix, crval, cdelt, logwave));
                         if (c1 > c2) {
                             int tmp = c1;
                             c1 = c2;
@@ -1267,8 +1362,10 @@ void QFitsCubeSpectrumViewer::paintEvent(QPaintEvent*) {
                     if ((sb != NULL) &&
                         (sb->getLineCont1()))
                     {
-                        c1 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineCont1Center() - sb->getLineCont1Width() - crpix) * cdelt - cdelt / 2.) + 0.5);
-                        c2 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineCont1Center() + sb->getLineCont1Width() - crpix) * cdelt + cdelt / 2.) + 0.5);
+//                        c1 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineCont1Center() - sb->getLineCont1Width() - crpix) * cdelt - cdelt / 2.) + 0.5);
+//                        c2 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineCont1Center() + sb->getLineCont1Width() - crpix) * cdelt + cdelt / 2.) + 0.5);
+                        c1 = wavelengthToPixel(pixwave(sb->getCubeCenter(QFV::Default) + sb->getLineCont1Center() - sb->getLineCont1Width() - 0.5, crpix, crval, cdelt, logwave));
+                        c2 = wavelengthToPixel(pixwave(sb->getCubeCenter(QFV::Default) + sb->getLineCont1Center() + sb->getLineCont1Width() + 0.5, crpix, crval, cdelt, logwave));
                         if (c1 > c2) {
                             int tmp = c1;
                             c1 = c2;
@@ -1287,8 +1384,10 @@ void QFitsCubeSpectrumViewer::paintEvent(QPaintEvent*) {
                     if ((sb != NULL) &&
                         (sb->getLineCont2()))
                     {
-                        c1 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineCont2Center() - sb->getLineCont2Width() - crpix) * cdelt - cdelt / 2.) + 0.5);
-                        c2 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineCont2Center() + sb->getLineCont2Width() - crpix) * cdelt + cdelt / 2.) + 0.5);
+//                        c1 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineCont2Center() - sb->getLineCont2Width() - crpix) * cdelt - cdelt / 2.) + 0.5);
+//                        c2 = (int)(wavelengthToPixel(crval + (sb->getCubeCenter(QFV::Default) + sb->getLineCont2Center() + sb->getLineCont2Width() - crpix) * cdelt + cdelt / 2.) + 0.5);
+                        c1 = wavelengthToPixel(pixwave(sb->getCubeCenter(QFV::Default) + sb->getLineCont2Center() - sb->getLineCont2Width() - 0.5, crpix, crval, cdelt, logwave));
+                        c2 = wavelengthToPixel(pixwave(sb->getCubeCenter(QFV::Default) + sb->getLineCont2Center() + sb->getLineCont2Width() + 0.5, crpix, crval, cdelt, logwave));
                         if (c1 > c2) {
                             int tmp = c1;
                             c1 = c2;
@@ -1330,7 +1429,7 @@ void QFitsCubeSpectrumViewer::paintEvent(QPaintEvent*) {
         drawSelection(&p);
 
         // paint cursor position information
-        p.setWorldMatrix(origMatrix);
+        p.setWorldTransform(origMatrix);
         p.setClipping(false);
         p.drawText(QRect(0, 0, width() - margin, fh), Qt::AlignRight|Qt::AlignBottom, posinfo);
 
@@ -1338,6 +1437,11 @@ void QFitsCubeSpectrumViewer::paintEvent(QPaintEvent*) {
     }
 }
 
+//void QFitsCubeSpectrumViewer::resizeEvent(QResizeEvent *e) {
+//    QFitsView1D::resizeEvent(e);
+//    fh = (int)((double)fh * 0.6);
+//}
+
 void QFitsCubeSpectrumViewer::mouseMoveEvent(QMouseEvent *e) {
     if (!hasFocus()) {
         setFocus();
@@ -1433,14 +1537,15 @@ void QFitsCubeSpectrumViewer::keyReleaseEvent(QKeyEvent *e) {
                     if (width > 0) {
                         sb->setCubeMode(DisplayCubeLinemap);
                         sb->setLineWidth(width, QFV::Default);
-                        sb->setCubeCenter(crpix + ((leftKey + rightKey) / 2.0 - crval) / cdelt,
-                                          QFV::Default);
+//                        sb->setCubeCenter(crpix + ((leftKey + rightKey) / 2.0 - crval) / cdelt,
+//                                          QFV::Default);
                     } else {
                         sb->setCubeMode(DisplayCubeSingle);
                         sb->setLineWidth(1, QFV::Default);
-                        sb->setCubeCenter(crpix + ((leftKey + rightKey) / 2.0 - crval) / cdelt,
-                                          QFV::Default);
+//                        sb->setCubeCenter(crpix + ((leftKey + rightKey) / 2.0 - crval) / cdelt,
+//                                          QFV::Default);
                     }
+                    sb->setCubeCenter(wavepix((leftKey + rightKey) / 2., crpix, crval, cdelt, f->isAxisLog(3)), QFV::Default);
                     break;
                 default:
                     break;
@@ -1454,7 +1559,8 @@ void QFitsCubeSpectrumViewer::keyReleaseEvent(QKeyEvent *e) {
                     case Qt::Key_X:
                         if (width > 0) {
                             sb->setLineCont1(true);
-                            sb->setLineCont1Center((int)(crpix + ((leftKey + rightKey) / 2.0 - crval) / cdelt - sb->getCubeCenter(QFV::Default)) + 0.5);
+//                            sb->setLineCont1Center((int)(crpix + ((leftKey + rightKey) / 2.0 - crval) / cdelt - sb->getCubeCenter(QFV::Default)) + 0.5);
+                            sb->setLineCont1Center(wavepix((leftKey + rightKey) / 2.0, crpix, crval, cdelt, f->isAxisLog(3)) - sb->getCubeCenter(QFV::Default));
                             sb->setLineCont1Width(width);
                         } else {
                             sb->setLineCont1(false);
@@ -1463,7 +1569,8 @@ void QFitsCubeSpectrumViewer::keyReleaseEvent(QKeyEvent *e) {
                     case Qt::Key_V:
                         if (width > 0) {
                             sb->setLineCont2(true);
-                            sb->setLineCont2Center((int)(crpix + ((leftKey + rightKey) / 2.0 - crval) / cdelt - sb->getCubeCenter(QFV::Default)) + 0.5);
+//                            sb->setLineCont2Center((int)(crpix + ((leftKey + rightKey) / 2.0 - crval) / cdelt - sb->getCubeCenter(QFV::Default)) + 0.5);
+                            sb->setLineCont2Center(wavepix((leftKey + rightKey) / 2.0, crpix, crval, cdelt, f->isAxisLog(3)) - sb->getCubeCenter(QFV::Default));
                             sb->setLineCont2Width(width);
                         } else {
                             sb->setLineCont2(false);
@@ -1635,7 +1742,6 @@ void QFitsCubeSpectrumViewer::setXRange(const double &xMin, const double &xMax)
     }
 
     fitsMainWindow->spectrum->spectrumRangeChanged(tmp_xmin, tmp_xmax);
-    fitsMainWindow->mytoolbar->setRangeValues();
 
     update();
 }
