@@ -31,7 +31,7 @@ QFitsGrid::QFitsGrid(QFitsMainView *parent, int howmany)
     }
     grid = new QGridLayout(this);
     grid->setSpacing(1);
-    grid->setMargin(1);
+    grid->setContentsMargins(1, 1, 1, 1);
     grid->addWidget(widgets[0], 0, 0);
     widgets[0]->show();
     grid->addWidget(widgets[1], 0, 1);
@@ -485,6 +485,7 @@ void QFitsMainView::saveImage() {
 
 void QFitsMainView::saveFits() {
     getCurrentBuffer()->saveFits();
+    fitsMainWindow->dpuserVar(getCurrentBufferIndex());
 }
 
 void QFitsMainView::printImage() {
@@ -592,7 +593,7 @@ void QFitsMainView::keyPressEvent(QKeyEvent *e) {
 }
 
 void QFitsMainView::playTetris() {
-    qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
+    srand(QTime(0,0,0).secsTo(QTime::currentTime()));
     TetrixWindow tetris;
     tetris.setWindowModality(Qt::NonModal);
     tetris.exec();
