@@ -31,36 +31,15 @@
 #endif
 #include "QFitsWidgetWiregrid.h"
 
-#include "resources/fileopen.xpm"
-#include "resources/reload.xpm"
-#include "resources/filesave.xpm"
-#include "resources/editcopy.xpm"
-#include "resources/fileprint.xpm"
-#include "resources/flip.xpm"
-#include "resources/flop.xpm"
-#include "resources/rotate.xpm"
-#include "resources/rotate2.xpm"
-#include "resources/zoomin.xpm"
-#include "resources/zoomout.xpm"
-#include "resources/movieplay.xpm"
-#include "resources/moviepause.xpm"
-#include "resources/plus.xpm"
-#include "resources/minus.xpm"
-#include "resources/divide.xpm"
-#include "resources/multiply.xpm"
-#include "resources/power.xpm"
-#include "resources/convolve.xpm"
-
 QFitsToolBar::QFitsToolBar(QFitsMainWindow *parent) :
                                        QWidget(dynamic_cast<QWidget*>(parent)) {
     setFocusPolicy(Qt::StrongFocus);
 
-    setMinimumSize(QSize(950, 60));
-    setFixedSize(890, 60);
+//    setMinimumSize(QSize(950, 60));
+//    setFixedSize(890, 60);
     updateAll = true;
 
     buttonOpen = new QToolButton(this);
-    buttonOpen->setIcon(QPixmap(fileopen));
     buttonOpen->setIcon(QIcon(":/open.svg"));
 //    buttonOpen->setIcon(buttonOpen->style()->standardIcon(QStyle::SP_DialogOpenButton));
     buttonOpen->setGeometry(QRect(0, 0, 30, 30));
@@ -77,7 +56,6 @@ QFitsToolBar::QFitsToolBar(QFitsMainWindow *parent) :
             parent, SLOT(reloadImage()));
 
     buttonSave = new QToolButton(this);
-    buttonSave->setIcon(QPixmap(filesave));
     buttonSave->setIcon(QIcon(":/save.svg"));
     buttonSave->setGeometry(QRect(60, 0, 30, 30));
     buttonSave->setToolTip("Save file");
@@ -85,7 +63,6 @@ QFitsToolBar::QFitsToolBar(QFitsMainWindow *parent) :
             parent->main_view, SLOT(saveImage()));
 
     buttonCopy = new QToolButton(this);
-    buttonCopy->setIcon(QPixmap(editcopy));
     buttonCopy->setIcon(QIcon(":/copy.svg"));
     buttonCopy->setGeometry(QRect(0, 30, 30, 30));
     buttonCopy->setToolTip("Copy current buffer to clipboard");
@@ -93,7 +70,6 @@ QFitsToolBar::QFitsToolBar(QFitsMainWindow *parent) :
             parent, SLOT(copyImage()));
 
     buttonPrint = new QToolButton(this);
-    buttonPrint->setIcon(QPixmap(fileprint));
     buttonPrint->setIcon(QIcon(":/print.svg"));
     buttonPrint->setGeometry(QRect(30, 30, 30, 30));
     buttonPrint->setToolTip("Print current buffer");
@@ -190,14 +166,14 @@ QFitsToolBar::QFitsToolBar(QFitsMainWindow *parent) :
 
 //#ifdef HAS_VTK
     buttonZoomIn_3Dwire = new QToolButton(this);
-    buttonZoomIn_3Dwire->setIcon(QPixmap(zoomin_icon));
+    buttonZoomIn_3Dwire->setIcon(QIcon(":/zoomin.svg"));
     buttonZoomIn_3Dwire->setGeometry(QRect(255, 30, 30, 30));
     buttonZoomIn_3Dwire->setToolTip("Zoom in");
     connect(buttonZoomIn_3Dwire, SIGNAL(clicked()),
             this, SLOT(incZoom_3Dwire()));
 
     buttonZoomOut_3Dwire = new QToolButton(this);
-    buttonZoomOut_3Dwire->setIcon(QPixmap(zoomout_icon));
+    buttonZoomOut_3Dwire->setIcon(QIcon(":/zoomout.svg"));
     buttonZoomOut_3Dwire->setGeometry(QRect(115, 30, 30, 30));
     buttonZoomOut_3Dwire->setToolTip("Zoom out");
     connect(buttonZoomOut_3Dwire, SIGNAL(clicked()),
@@ -296,38 +272,6 @@ QFitsToolBar::QFitsToolBar(QFitsMainWindow *parent) :
     connect(imageMaxValue, SIGNAL(returnPressed()),
             this, SLOT(setImageLimitsManual()));
 
-    specChannelMinXValue = new QSpinBox(this);
-    specChannelMinXValue->setGeometry(QRect(330, 30, 80, 30));
-    specChannelMinXValue->setKeyboardTracking(false);
-    specChannelMinXValue->setToolTip("Spectrum min channel");
-    connect(specChannelMinXValue, SIGNAL(valueChanged(int)),
-            this, SLOT(setSpecChannelMinXVal(int)));
-    connect(specChannelMinXValue, SIGNAL(editingFinished()),
-            this, SLOT(setSpecChannelMinXVal2()));
-
-    specChannelMaxXValue = new QSpinBox(this);
-    specChannelMaxXValue->setGeometry(QRect(410, 30, 80, 30));
-    specChannelMaxXValue->setKeyboardTracking(false);
-    specChannelMaxXValue->setToolTip("Spectrum max channel");
-    connect(specChannelMaxXValue, SIGNAL(valueChanged(int)),
-            this, SLOT(setSpecChannelMaxXVal(int)));
-    connect(specChannelMaxXValue, SIGNAL(editingFinished()),
-            this, SLOT(setSpecChannelMaxXVal2()));
-
-    specPhysMinXValue = new QLineEdit(this);
-    specPhysMinXValue->setGeometry(QRect(230, 30, 100, 30));
-    specPhysMinXValue->setToolTip("Spectrum min wavelength");
-    specPhysMinXValue->setValidator(new QDoubleValidator(specPhysMinXValue));
-    connect(specPhysMinXValue, SIGNAL(returnPressed()),
-            this, SLOT(setSpecPhysicalMinXVal()));
-
-    specPhysMaxXValue = new QLineEdit(this);
-    specPhysMaxXValue->setGeometry(QRect(490, 30, 100, 30));
-    specPhysMaxXValue->setToolTip("Spectrum max wavelength");
-    specPhysMaxXValue->setValidator(new QDoubleValidator(specPhysMaxXValue));
-    connect(specPhysMaxXValue, SIGNAL(returnPressed()),
-            this, SLOT(setSpecPhysicalMaxXVal()));
-
     specPhysMinYValue = new QLineEdit(this);
     specPhysMinYValue->setGeometry(QRect(310, 0, 100, 30));
     specPhysMinYValue->setToolTip("Spectrum min value");
@@ -341,10 +285,7 @@ QFitsToolBar::QFitsToolBar(QFitsMainWindow *parent) :
             this, SLOT(setSpecYLimitsManual()));
 
     setTabOrder(specPhysMinYValue, specPhysMaxYValue);
-    setTabOrder(specPhysMaxYValue, specPhysMinXValue);
-    setTabOrder(specPhysMinXValue, specChannelMinXValue);
-    setTabOrder(specChannelMinXValue, specChannelMaxXValue);
-    setTabOrder(specChannelMaxXValue, specPhysMaxXValue);
+//    setTabOrder(specPhysMaxYValue, specPhysMinXValue);
 
     imredPlus = new QToolButton(this);
 //    imredPlus->setIcon(QPixmap(plus_xpm));
@@ -454,6 +395,84 @@ void QFitsToolBar::keyPressEvent( QKeyEvent *e ) {
     fitsMainWindow->main_view->keyPressEvent(e);
 }
 
+void QFitsToolBar::resizeEvent(QResizeEvent *e) {
+    int h = e->size().height() / 2;
+    int x = 0;
+// TODO QT6
+//    int cw = imageScaleMethod->fontMetrics().horizontalAdvance("square root") * 1.7;
+		int cw = imageScaleMethod->fontMetrics().boundingRect("square root").width() * 1.7;
+
+    // upper row
+    buttonOpen->setGeometry(QRect(x, 0, h, h)); x += h;
+    buttonReload->setGeometry(QRect(x, 0, h, h)); x += h;
+    buttonSave->setGeometry(QRect(x, 0, h, h)); x += h + h/2;
+
+    buttonFlip->setGeometry(QRect(x, 0, h, h)); x += h;
+    buttonRotateCCW->setGeometry(QRect(x, 0, h, h)); x += h;
+    comboRotate->setGeometry(QRect(x, 0, 3*h, h)); x += 3*h;
+    buttonRotateCW->setGeometry(QRect(x, 0, h, h)); x += h;
+    buttonFlop->setGeometry(QRect(x, 0, h, h)); x += h + h/2;
+
+    imageMinValue->setGeometry(QRect(x, 0, cw, h));
+    specPhysMinYValue->setGeometry(QRect(x, 0, cw, h)); x += cw;
+    imageMaxValue->setGeometry(QRect(x, 0, cw, h));
+    specPhysMaxYValue->setGeometry(QRect(x, 0, cw, h)); x += cw + h/2;
+
+    cubeImageSlice->setGeometry(QRect(x, 0, 3*h, h));
+    limitZCheckBox->setGeometry(x, 0, limitZCheckBox->width(), h);
+    limitSpinboxZ->setGeometry(x + limitZCheckBox->width(), 0, 6*h-limitZCheckBox->width(), h); x += 3*h;
+    cubeMode->setGeometry(QRect(x, 0, 3*h, h)); x += 3*h + h/2;
+    alphaMultiplierLabel->setGeometry(QRect(x, 0, alphaMultiplierLabel->width(), h));
+    alphaMultiplierSlider->setGeometry(QRect(x + alphaMultiplierLabel->width(), 0, 6*h-alphaMultiplierLabel->width(), h));
+
+    imredPlus->setGeometry(QRect(x, 0, h, h)); x += h;
+    imredMinus->setGeometry(QRect(x, 0, h, h)); x += h;
+    imredMul->setGeometry(QRect(x, 0, h, h)); x += h;
+    imredDiv->setGeometry(QRect(x, 0, h, h)); x += h;
+    imredPow->setGeometry(QRect(x, 0, h, h)); x += h;
+    imredConv->setGeometry(QRect(x, 0, h, h));
+
+    buttonZoomIn->setGeometry(QRect(255, 30, 30, 30));
+
+    // lower row
+    x = 0;
+    buttonCopy->setGeometry(QRect(x, h, h, h)); x += h;
+    buttonPrint->setGeometry(QRect(h, h, h, h)); x += h;
+
+    x += 2*h;
+    buttonZoomOut->setGeometry(QRect(x, h, h, h));
+    buttonZoomOut_3Dwire->setGeometry(QRect(x, h, h, h)); x += h;
+    comboZoom->setGeometry(QRect(x, h, 4*h, h));
+    comboZoom_3Dwire->setGeometry(QRect(x, h, 4*h, h)); x += 4*h;
+    buttonZoomIn->setGeometry(QRect(x, h, h, h));
+    buttonZoomIn_3Dwire->setGeometry(QRect(x, h, h, h)); x += h + h;
+
+    imageScaleRange->setGeometry(QRect(x, h, cw, h)); x += cw;
+    imageScaleMethod->setGeometry(QRect(x, h, cw, h)); x += cw + h/2;
+
+    movieSlider->setGeometry(QRect(x, h, 5*h, h)); x += 5*h;
+    movieButton->setGeometry(QRect(x, h, h, h)); x += h/2;
+    drawSliceFramesCheckBox->setGeometry(QRect(x, h, 6*h, h));
+
+// set Icon Size of all ToolButtons
+    QSize iconSize = QSize(h/1.8, h/1.8);
+//    QFont theFont = imageMinValue->createStandardContextMenu()->font();
+//    theFont.setPixelSize(iconSize.height()/2);
+    QObjectList buttons = children();
+    for (int i = 0; i < buttons.count(); i++) {
+        QToolButton *button = dynamic_cast<QToolButton*>(buttons.at(i));
+        if (button != NULL) {
+            button->setIconSize(iconSize);
+        } else {
+//            QWidget *widget = dynamic_cast<QWidget*>(buttons.at(i));
+//            if (widget != NULL) {
+//                widget->setFont(theFont);
+//                widget->update();
+//            }
+        }
+    }
+}
+
 QFitsSingleBuffer* QFitsToolBar::getMyBuffer() {
     if (fitsMainWindow != NULL) {
         QFitsSingleBuffer *sb = fitsMainWindow->getActualSB();
@@ -561,19 +580,11 @@ void QFitsToolBar::showControls() {
 void QFitsToolBar::enableControlsView1D(bool enable) {
     specPhysMinYValue->setEnabled(enable);
     specPhysMaxYValue->setEnabled(enable);
-    specPhysMinXValue->setEnabled(enable);
-    specPhysMaxXValue->setEnabled(enable);
-    specChannelMinXValue->setEnabled(enable);
-    specChannelMaxXValue->setEnabled(enable);
 
     if (!enable) {
         updateAll = false;
         specPhysMinYValue->setText("");
         specPhysMaxYValue->setText("");
-        specPhysMinXValue->setText("");
-        specPhysMaxXValue->setText("");
-        specChannelMinXValue->setValue(0);
-        specChannelMaxXValue->setValue(0);
         updateAll = true;
     }
 }
@@ -592,11 +603,6 @@ void QFitsToolBar::enableControlsView2D(bool enable) {
     }
 }
 
-void QFitsToolBar::enableControlsPhysX(bool enable) {
-    specPhysMinXValue->setEnabled(enable);
-    specPhysMaxXValue->setEnabled(enable);
-}
-
 void QFitsToolBar::enableControlsArithmetic(bool enable) {
     imredPlus->setEnabled(enable);
     imredMinus->setEnabled(enable);
@@ -609,12 +615,8 @@ void QFitsToolBar::enableControlsArithmetic(bool enable) {
 void QFitsToolBar::setPaletteControls(QPalette &pal) {
     specPhysMinYValue->setPalette(pal);
     specPhysMaxYValue->setPalette(pal);
-    specPhysMinXValue->setPalette(pal);
-    specPhysMaxXValue->setPalette(pal);
     imageMinValue->setPalette(pal);
     imageMaxValue->setPalette(pal);
-    specChannelMinXValue->setPalette(pal);
-    specChannelMaxXValue->setPalette(pal);
 }
 
 void QFitsToolBar::updateImageMinMax(QFitsBaseBuffer *bb) {
@@ -731,10 +733,6 @@ void QFitsToolBar::showControlsScaling(bool show) {
             imageScaleRange->hide();
             imageScaleMethod->hide();
 
-            specChannelMinXValue->show();
-            specChannelMaxXValue->show();
-            specPhysMinXValue->show();
-            specPhysMaxXValue->show();
             specPhysMinYValue->show();
             specPhysMaxYValue->show();
         } else {
@@ -744,10 +742,6 @@ void QFitsToolBar::showControlsScaling(bool show) {
             imageScaleRange->show();
             imageScaleMethod->show();
 
-            specChannelMinXValue->hide();
-            specChannelMaxXValue->hide();
-            specPhysMinXValue->hide();
-            specPhysMaxXValue->hide();
             specPhysMinYValue->hide();
             specPhysMaxYValue->hide();
         }
@@ -757,10 +751,6 @@ void QFitsToolBar::showControlsScaling(bool show) {
         imageScaleRange->hide();
         imageScaleMethod->hide();
 
-        specChannelMinXValue->hide();
-        specChannelMaxXValue->hide();
-        specPhysMinXValue->hide();
-        specPhysMaxXValue->hide();
         specPhysMinYValue->hide();
         specPhysMaxYValue->hide();
     }
@@ -935,31 +925,6 @@ void QFitsToolBar::updateValues() {
     if (sb != NULL) {
         QReadLocker locker(&buffersLock);
         if (sb->Naxis(0) == 1) {
-            dpuserType *dp = sb->getDpData();
-            if ((dp->type == typeFits) &&
-                (dp->fvalue != NULL))
-            {
-                Fits *f = dp->fvalue;
-                updateAll = false;
-                if (f->getCDELT(1) >= 0) {
-                    specChannelMinXValue->setMinimum(1);
-                    specChannelMinXValue->setMaximum(f->Naxis(1) - 1);
-
-                    specChannelMaxXValue->setMinimum(2);
-                    specChannelMaxXValue->setMaximum(f->Naxis(1));
-                } else {
-                    specChannelMinXValue->setMinimum(2);
-                    specChannelMinXValue->setMaximum(f->Naxis(1));
-
-                    specChannelMaxXValue->setMinimum(1);
-                    specChannelMaxXValue->setMaximum(f->Naxis(1) - 1);
-                }
-                specChannelMinXValue->setValue(sb->getSpecChannelMinX());
-                specChannelMaxXValue->setValue(sb->getSpecChannelMaxX());
-                specPhysMinXValue->setText(QString::number(sb->getSpecPhysMinX()));
-                specPhysMaxXValue->setText(QString::number(sb->getSpecPhysMaxX()));
-                updateAll = true;
-            }
         } else if (sb->Naxis(0) == 2) {
         } else if (sb->Naxis(0) == 3) {
 //#ifdef HAS_VTK
@@ -1054,141 +1019,6 @@ void QFitsToolBar::setImageLimitsScaleRange(int selected) {
     }
 }
 
-void QFitsToolBar::setSpecChannelMinXVal(int minChannelVal) {
-    if (updateAll) {
-        QFitsSingleBuffer *sb = getMyBuffer();
-        dpuserType *dp = sb->getDpData();
-        if ((sb != NULL) &&
-            (dp->type == typeFits) &&
-            (dp->fvalue != NULL))
-        {
-            QReadLocker locker(&buffersLock);
-
-            sb->setSpecChannelMinX(minChannelVal);
-            specPhysMinXValue->setText(QString::number(sb->getSpecPhysMinX()));
-
-            if (sb->getState() != NULL) {
-                sb->getState()->setXRange(sb->getSpecPhysMinX(), sb->getSpecPhysMaxX());
-            }
-        }
-    }
-}
-
-// in fact the same as updateSpecChannelMinXVal(), but this one is called when
-// specChannelMinXValue looses focus and the value has been changed
-void QFitsToolBar::setSpecChannelMinXVal2() {
-    if (updateAll) {
-        setSpecChannelMinXVal(specChannelMinXValue->value());
-    }
-}
-
-void QFitsToolBar::setSpecChannelMaxXVal(int maxChannelVal) {
-    if (updateAll) {
-        QFitsSingleBuffer *sb = getMyBuffer();
-        if ((sb != NULL) &&
-            (sb->getDpData()->type == typeFits) &&
-            (sb->getDpData()->fvalue != NULL))
-        {
-            QReadLocker locker(&buffersLock);
-            sb->setSpecChannelMaxX(maxChannelVal);
-            specPhysMaxXValue->setText(QString::number(sb->getSpecPhysMaxX()));
-
-            if (sb->getState() != NULL) {
-                sb->getState()->setXRange(sb->getSpecPhysMinX(), sb->getSpecPhysMaxX());
-            }
-        }
-    }
-}
-
-// in fact the same as updateSpecChannelMaxXVal(), but this one is called when
-// specChannelMaxXValue looses focus and the value has been changed
-void QFitsToolBar::setSpecChannelMaxXVal2() {
-    if (updateAll) {
-        setSpecChannelMaxXVal(specChannelMaxXValue->value());
-    }
-}
-
-void QFitsToolBar::setSpecPhysicalMinXVal() {
-    if (updateAll) {
-        QFitsSingleBuffer *sb = getMyBuffer();
-        if ((sb != NULL) &&
-            (sb->getDpData()->type == typeFits) &&
-            (sb->getDpData()->fvalue != NULL) &&
-            specPhysMinXValue->hasFocus() &&
-            !specChannelMinXValue->hasFocus())
-        {
-            QReadLocker locker(&buffersLock);
-            sb->setSpecPhysMinX(specPhysMinXValue->text().toDouble());
-            specChannelMinXValue->setValue(sb->getSpecChannelMinX());
-
-            if (sb->getState() != NULL) {
-                sb->getState()->setXRange(sb->getSpecPhysMinX(), sb->getSpecPhysMaxX());
-            }
-        }
-    }
-}
-
-void QFitsToolBar::setSpecPhysicalMaxXVal() {
-    if (updateAll) {
-        QFitsSingleBuffer *sb = getMyBuffer();
-        if ((sb != NULL) &&
-            (sb->getDpData()->type == typeFits) &&
-            (sb->getDpData()->fvalue != NULL) &&
-            specPhysMaxXValue->hasFocus() &&
-            !specChannelMaxXValue->hasFocus())
-        {
-            QReadLocker locker(&buffersLock);
-            sb->setSpecPhysMaxX(specPhysMaxXValue->text().toDouble());
-            specChannelMaxXValue->setValue(sb->getSpecChannelMaxX());
-
-            if (sb->getState() != NULL) {
-                sb->getState()->setXRange(sb->getSpecPhysMinX(), sb->getSpecPhysMaxX());
-            }
-        }
-    }
-}
-
-void QFitsToolBar::setRangeValues() {
-    QFitsSingleBuffer *sb = getMyBuffer();
-
-    if ((sb != NULL) &&
-        (sb->getDpData()->type == typeFits) &&
-        (sb->getDpData()->fvalue != NULL))
-    {
-        updateAll = false;
-        if (!specChannelMinXValue->hasFocus()) {
-            specChannelMinXValue->setValue(sb->getSpecChannelMinX());
-        }
-        if (!specChannelMaxXValue->hasFocus()) {
-            specChannelMaxXValue->setValue(sb->getSpecChannelMaxX());
-        }
-        if (!specPhysMinXValue->hasFocus()) {
-            specPhysMinXValue->setText(QString::number(sb->getSpecPhysMinX()));
-        }
-        if (!specPhysMaxXValue->hasFocus()) {
-            specPhysMaxXValue->setText(QString::number(sb->getSpecPhysMaxX()));
-        }
-        updateAll = true;
-    }
-}
-
-void QFitsToolBar::blankRangeValues() {
-    updateAll = false;
-    if (!specChannelMinXValue->hasFocus()) {
-        specChannelMinXValue->setValue(0);
-    }
-    if (!specChannelMaxXValue->hasFocus()) {
-        specChannelMaxXValue->setValue(0);
-    }
-    if (!specPhysMinXValue->hasFocus()) {
-        specPhysMinXValue->setText(QString::number(0));
-    }
-    if (!specPhysMaxXValue->hasFocus()) {
-        specPhysMaxXValue->setText(QString::number(0));
-    }
-    updateAll = true;
-}
-
 void QFitsToolBar::createArithPopupMenu(int arithmetics, QMenu *arithPopupMenu) {
     requestedArithmetics = arithmetics;
     QAction *action = NULL;
@@ -1312,7 +1142,6 @@ void QFitsToolBar::doArithmetics(QAction *action) {
 }
 
 ActionComboBox::ActionComboBox(QWidget *parent): QComboBox(parent) {
-
 }
 
 void ActionComboBox::setActions(QActionGroup *actions) {
