@@ -20,7 +20,8 @@
 //         QFitsView1D
 //------------------------------------------------------------------------------
 QFitsView1D::QFitsView1D(QWidget *parent) : QFitsBaseView(parent) {
-    m.setMatrix(1, 0, 0, 1, 0, 0);
+//    m.setMatrix(1, 0, 0, 1, 0, 0);
+    m.reset();
     plotwidth = width();
     plotheight = height();
     margin = 10;
@@ -30,6 +31,7 @@ QFitsView1D::QFitsView1D(QWidget *parent) : QFitsBaseView(parent) {
     y2data = NULL;
     ndata = 0;
     n2data = 0;
+    logwave = false;
     autoScaleSpectrum = true;
     selectstart = QPoint(-1, -1);
     selectend = QPoint(-1, -1);
@@ -92,6 +94,11 @@ void QFitsView1D::createPopupMenu() {
     copyMBbufferAct->setStatusTip(tr("Copies this extension to a new buffer"));
     connect(copyMBbufferAct, SIGNAL(triggered()), this, SLOT(copyMBbufferSlot()));
 
+#ifdef HAS_PLOTTING
+    plotSpecAct = new QAction("Send spectrum to plotting tool", this);
+    connect(plotSpecAct, SIGNAL(triggered()), this, SLOT(sendToPlotterSlot()));
+#endif /* HAS_PLOTTING */
+
     // fill popup menu
     rightPopupMenu->addAction(zoomOutAct);
     rightPopupMenu->addAction(plotStyleLinesAct);
@@ -102,7 +109,10 @@ void QFitsView1D::createPopupMenu() {
     rightPopupMenu->addSeparator();
     rightPopupMenu->addAction(saveMBbufferAct);
     rightPopupMenu->addAction(copyMBbufferAct);
+#ifdef HAS_PLOTTING
     rightPopupMenu->addSeparator();
+    rightPopupMenu->addAction(plotSpecAct);
+#endif /* HAS_PLOTTING */
 }
 
 int QFitsView1D::calcPos(QPoint pos) {
@@ -114,13 +124,14 @@ int QFitsView1D::calcPos(QPoint pos) {
 
     QFitsSingleBuffer *sb = getMyBuffer();
     if (sb != NULL) {
+        dpString fstring;
         double  value   = 0.0,
                 xmin    = sb->getSpecPhysMinX(),
                 xmax    = sb->getSpecPhysMaxX(),
                 xscale  = (double)(plotwidth) / (xmax - xmin),
                 xpos    = (pos.x() - fw - margin) / xscale + xmin;
 
-        posInt = (int)(crpix + (xpos - crval) / cdelt + 0.5);
+        posInt = wavepix(xpos, crpix, crval, cdelt, logwave); // (int)(crpix + (xpos - crval) / cdelt + 0.5);
 
         if ((posInt > 0) && (posInt <= ndata)) {
             if (cdelt >= 0) {
@@ -131,10 +142,11 @@ int QFitsView1D::calcPos(QPoint pos) {
         }
 
         if ((posInt > 0) && (posInt <= ndata)) {
-            posinfo.sprintf("#%i (%g, %g)", posInt, xpos, value);
+            fstring.sprintf("#%i (%g, %g)", posInt, xpos, value);
         } else {
-            posinfo.sprintf("    (%g, -)", xpos);
+            fstring.sprintf("    (%g, -)", xpos);
         }
+        posinfo = fstring.c_str();
     }
 
     return posInt;
@@ -172,15 +184,30 @@ void QFitsView1D::setData() {
         crval = spectrum->getCRVAL(axis);
         crpix = spectrum->getCRPIX(axis);
         cdelt = spectrum->getCDELT(axis);
+        logwave = spectrum->isAxisLog(axis);
 
         if (cdelt > 0.0) {
             for (i = 0; i < ndata; i++) {
-                xdata[i] = crval + ((double)(i+1) - crpix) * cdelt;
+                xdata[i] = pixwave(i+1, crpix, crval, cdelt, logwave);
+/*
+                if (logwave) {
+                    xdata[i] = crval * exp(((double)(i+1) - crpix) * cdelt / crval); // following Greisen et al, 2006, EQ 4 & 5
+                } else {
+                    xdata[i] = crval + ((double)(i+1) - crpix) * cdelt;
+                }
+*/
                 ydata[i] = spectrum->ValueAt(i);
             }
         } else {
             for (i = 0, j = ndata-1; i < ndata; i++, j--) {
-                xdata[j] = crval + ((double)(i+1) - crpix) * cdelt;
+                xdata[i] = pixwave(i+1, crpix, crval, cdelt, logwave);
+/*
+                if (logwave) {
+                    xdata[i] = crval * exp(((double)(i+1) - crpix) * cdelt / crval); // following Greisen et al, 2006, EQ 4 & 5
+                } else {
+                    xdata[i] = crval + ((double)(i+1) - crpix) * cdelt;
+                }
+*/
                 ydata[j] = spectrum->ValueAt(i);
             }
         }
@@ -278,7 +305,7 @@ void QFitsView1D::setXRange(const double &xMin, const double &xMax) {
     if (sb != NULL) {
         sb->setSpecPhysMinX(tmp_xmin);
         sb->setSpecPhysMaxX(tmp_xmax);
-        fitsMainWindow->updateToolbar();
+//@        fitsMainWindow->updateToolbar();
         if (sb->isMultiBufferChild() && sb->getMyMultiBuffer()->hasCubeSpecHomogenSB()) {
             sb->getMyMultiBuffer()->update();
         } else {
@@ -424,7 +451,7 @@ void QFitsView1D::paintEvent(QPaintEvent *e) {
 
         pa.drawText(QRect(0, 0, width() - margin, fh), Qt::AlignRight|Qt::AlignBottom, posinfo);
         drawAxis(&pa, ymin, ymax);
-        pa.setWorldMatrix(m);
+        pa.setWorldTransform(m);
 
         QPolygon array;
         int j = calcPolyline(ndata, xdata, ydata, array, ymin, ymax);
@@ -488,12 +515,44 @@ void QFitsView1D::drawAxis(QPainter *pa, double ymin, double ymax) {
                     }
                 }
 
-                textrect = QRect((int)((start + i*delta-xmin) * xscale + 0.5) - 100 + fw + margin, height() - fh - 1, 201, fh);
-                if (limits.contains(pa->boundingRect(textrect, Qt::AlignHCenter|Qt::AlignBottom, text))) {
-                    pa->drawText(textrect, Qt::AlignHCenter|Qt::AlignBottom, text);
+                textrect = QRect((int)((start + i*delta-xmin) * xscale + 0.5) - 100 + fw + margin, height() - fh + 2, 201, fh);
+                if (limits.contains(pa->boundingRect(textrect, Qt::AlignHCenter|Qt::AlignTop, text))) {
+                    pa->drawText(textrect, Qt::AlignHCenter|Qt::AlignTop, text);
                     pa->drawLine((int)((start + i*delta-xmin) * xscale + 0.5) + fw + margin, height() - fh, (int)((start + i*delta-xmin) * xscale + 0.5) + fw + margin, height() - fh - 5);
                 }
             }
+            // label x-axis
+            Fits *f = sb->getCubeDisplay1D();
+            if (f != NULL) {
+                char value[81];
+                text.clear();
+                if (f->GetStringKey("CTYPE1", value)) {
+                    dpString type = value;
+                    type = type.left(4);
+                    if (type == "WAVE") text = QChar(955);
+                    else if (type.left(4) == "FREQ") text = QChar(957);
+                    else if (type.left(4) == "ENER") text = "E";
+                    else if (type.left(4) == "WAVN") text = QChar(954);
+                    else if (type.left(4) == "VRAD") text = "V";
+                    else if (type.left(4) == "VOPT") text = "Z";
+                    else if (type.left(4) == "ZOPT") text = "z";
+                    else if (type.left(4) == "AWAV") { text = QChar(955); text += "a"; }
+                    else if (type.left(4) == "VELO") text = "v";
+                    else if (type.left(4) == "BETA") text = QChar(946);
+                    else text = type.c_str();
+                }
+                if (f->GetStringKey("CUNIT1", value)) {
+                    if (!QString(value).trimmed().isEmpty()) {
+                        text += " [";
+                        text += QString(value).trimmed();
+                        text += "]";
+                    }
+                }
+                textrect = limits;
+                textrect.translate(0, -2);
+                pa->fillRect(pa->boundingRect(textrect, Qt::AlignHCenter|Qt::AlignBottom, text), QApplication::palette().window().color());
+                pa->drawText(textrect, Qt::AlignHCenter|Qt::AlignBottom, text);
+            }
 
             calcAxisScale(ymin, ymax, &n, &start, &delta);
             limits = QRect(0, 0, fw, plotheight + 2*margin);
@@ -521,6 +580,19 @@ void QFitsView1D::drawAxis(QPainter *pa, double ymin, double ymax) {
                     pa->drawLine(fw, height() - fh - margin - (int)((start + i*delta-ymin) * yscale + 0.5), fw + 5, height() - fh - margin - (int)((start + i*delta-ymin) * yscale + 0.5));
                 }
             }
+            if (f != NULL) {
+                char value[81];
+                if (f->GetStringKey("BUNIT", value)) {
+                    text = QString(value).trimmed();
+                    pa->rotate(-90.);
+                    textrect = QRect(0, 0, -height() + fh, fw);
+                    if (textrect.contains(pa->boundingRect(textrect, Qt::AlignHCenter|Qt::AlignTop, text))) {
+                        pa->fillRect(pa->boundingRect(textrect, Qt::AlignHCenter|Qt::AlignTop, text), QApplication::palette().window().color());
+                        pa->drawText(textrect, Qt::AlignHCenter|Qt::AlignTop, text);
+                    }
+                    pa->rotate(90.);
+                }
+            }
         }
     }
 }
@@ -660,7 +732,7 @@ void QFitsView1D::mouseReleaseEvent(QMouseEvent *m) {
                         start = round(start);
                         end = round(end);
                     }
-
+                    emit rangeChanged(start, end);
                     setXRange(start, end);
                     if ((mb != NULL) && mb->hasCubeSpecHomogenSB()) {
                         mb->updateHomogenSB();
@@ -688,9 +760,11 @@ void QFitsView1D::mouseReleaseEvent(QMouseEvent *m) {
 
 void QFitsView1D::resizeEvent(QResizeEvent *e) {
 //dp_debug("    QFitsView1D::resizeEvent()\n"); fflush(stdout);
+    fh = (int)(fontMetrics().height() * 2.5);
     fh = (int)(fontMetrics().height() * 1.5);
-    fw = fontMetrics().width("000000000");
-    m.setMatrix(1, 0, 0, -1, fw + margin, e->size().height()-fh-margin+1);
+    fw = fontMetrics().boundingRect("000000000").width();
+
+    m = QTransform(1, 0, 0, -1, fw + margin, e->size().height()-fh-margin);
     plotwidth = e->size().width() - fw - 2*margin;
     plotheight = e->size().height() - fh - 2*margin;
 }
@@ -703,14 +777,14 @@ void QFitsView1D::enterEvent(QEvent*) {
         QFitsSingleBuffer *sb = widget1D->getMyBuffer();
         if ((sb != NULL) && (sb->isMultiBufferChild())) {
             if (sb->isEmptyPrimary()) {
-                fitsMainWindow->mytoolbar->enableControlsView1D(false);
+//@                fitsMainWindow->mytoolbar->enableControlsView1D(false);
             } else {
-                fitsMainWindow->mytoolbar->enableControlsView1D(true);
+//@                fitsMainWindow->mytoolbar->enableControlsView1D(true);
                 if (getMyBuffer()->isMultiBufferChild() &&
                     (getMyBuffer()->getMyMultiBuffer()->getMarkedSingleBuffer() != NULL)) {
                     // do nothing
                 } else {
-                    fitsMainWindow->mytoolbar->setRangeValues();
+//@                    fitsMainWindow->mytoolbar->setRangeValues();
                 }
             }
         } else {
@@ -727,19 +801,20 @@ void QFitsView1D::leaveEvent (QEvent *e) {
         if ((sb != NULL) && (sb->isMultiBufferChild())) {
             // this belongs to a QFitsMultiBuffer
             if (sb->getMyMultiBuffer()->getMarkedSingleBuffer() == NULL) {
-                fitsMainWindow->mytoolbar->blankRangeValues();
+//@                fitsMainWindow->mytoolbar->blankRangeValues();
             }
         } else {
             // this belongs to a "real" QFitsSingleBuffer
         }
     }
-    posinfo.sprintf(" ");
+    posinfo = " ";
     update();
 }
 
 void QFitsView1D::zoomOut() {
     if (ndata != 0) {
         autoScaleSpectrum = true;
+        emit rangeChanged(xdata[0], xdata[ndata-1]);
         setXRange(xdata[0], xdata[ndata-1]);
         QFitsMultiBuffer  *mb = dynamic_cast<QFitsMultiBuffer*>(fitsMainWindow->getCurrentBuffer());
         if ((mb != NULL) && mb->hasCubeSpecHomogenSB()) {
@@ -799,7 +874,7 @@ void QFitsView1D::exportSpecAsciiSlot() {
             }
 
             for (int i = 0; i < f->Nelements(); i++) {
-                if (fprintf(fd, "%20.10f %20.10f\n", start + i * delta, f->ValueAt(i)) < 0) {
+                if (fprintf(fd, "%20.10f %20.10f\n", pixwave(i+1, f->getCRPIX(1), f->getCRVAL(1), f->getCDELT(1), logwave), f->ValueAt(i)) < 0) {
                     QMessageBox::warning(this, "QFitsView", "Could not write data");
                     fclose(fd);
                     return;
@@ -851,6 +926,12 @@ void QFitsView1D::copyMBbufferSlot() {
     }
 }
 
+void QFitsView1D::sendToPlotterSlot() {
+#ifdef HAS_PLOTTING
+    fitsMainWindow->plplotWindow->addBufferData(QString(fitsMainWindow->main_view->getCurrentBufferIndex().c_str()), -1, -1);
+#endif /* HAS_PLOTTING */
+}
+
 void QFitsView1D::calcAxisScale(const double min, const double max, int *n, double *start, double *delta) {
     int     increment   = 0,
             nn          = 0;
