@@ -29,9 +29,12 @@
 #include "dpuser_utils.h"
 #include "fitting.h"
 #include "QFitsToolBar.h"
+#include "dpstringlist.h"
+
+extern "C" {
 #include "resources/cursor_rotate.xpm"
 #include "resources/cursor_select.xpm"
-#include "dpstringlist.h"
+}
 
 QFitsView2D::QFitsView2D(QFitsScroller *parent, QFitsWidget2D* myWidget)
                         : QFitsBaseView(parent), myParent(myWidget)
@@ -46,6 +49,7 @@ QFitsView2D::QFitsView2D(QFitsScroller *parent, QFitsWidget2D* myWidget)
     setMouseTracking(true);
 
     mousemode = None;
+    distancemode = Pixels;
     setAcceptDrops(true);
 //    rightButtonMenuVisible = false;
 
@@ -83,6 +87,11 @@ void QFitsView2D::createPopupMenu() {
     copySpecBufferAct = new QAction("Copy spectrum to new buffer", this);
     connect(copySpecBufferAct, SIGNAL(triggered()), this, SLOT(copySpecBufferSlot()));
 
+#ifdef HAS_PLOTTING
+    plotSpecAct = new QAction("Send single spectrum to plotting tool", this);
+    connect(plotSpecAct, SIGNAL(triggered()), this, SLOT(plotSpecSlot()));
+#endif /* HAS_PLOTTING */
+
     copyImageBufferAct = new QAction("Copy image to new buffer", this);
     copyImageBufferAct->setStatusTip(tr("Copies image as it appears on screen"));
     connect(copyImageBufferAct, SIGNAL(triggered()), this, SLOT(copyImageBufferSlot()));
@@ -111,6 +120,14 @@ void QFitsView2D::createPopupMenu() {
     radialProfileAct = new QAction("Radial profile from this position", this);
     connect(radialProfileAct, SIGNAL(triggered()), this, SLOT(radialProfileSlot()));
 
+#ifdef HAS_PLOTTING
+    plotHorizontalAct = new QAction("Send horizontal cut to plotting tool", this);
+    connect(plotHorizontalAct, SIGNAL(triggered()), this, SLOT(plotHorizontalSlot()));
+
+    plotVerticalAct = new QAction("Send vertical cut to plotting tool", this);
+    connect(plotVerticalAct, SIGNAL(triggered()), this, SLOT(plotVerticalSlot()));
+#endif /* HAS_PLOTTING */
+
     lockPosAct = new QAction("Lock position", this);
     connect(lockPosAct, SIGNAL(triggered()), this, SLOT(lockPosSlot()));
 
@@ -121,6 +138,9 @@ void QFitsView2D::createPopupMenu() {
     rightPopupMenu->addAction(exportSpecAsciiAct);
     rightPopupMenu->addAction(exportSpecFitsAct);
     rightPopupMenu->addAction(copySpecBufferAct);
+#ifdef HAS_PLOTTING
+    rightPopupMenu->addAction(plotSpecAct);
+#endif /* HAS_PLOTTING */
     rightPopupMenu->addSeparator();
     rightPopupMenu->addAction(saveImageFitsAct);
     rightPopupMenu->addAction(copyImageBufferAct);
@@ -133,6 +153,10 @@ void QFitsView2D::createPopupMenu() {
     rightPopupMenu->addAction(loadMarkersAct);
     rightPopupMenu->addSeparator();
     rightPopupMenu->addAction(radialProfileAct);
+#ifdef HAS_PLOTTING
+    rightPopupMenu->addAction(plotHorizontalAct);
+    rightPopupMenu->addAction(plotVerticalAct);
+#endif /* HAS_PLOTTING */
     rightPopupMenu->addSeparator();
     rightPopupMenu->addAction(lockPosAct);
     rightPopupMenu->addAction(useStdStarAct);
@@ -156,6 +180,12 @@ void QFitsView2D::copySpecBufferSlot() {
     setMouseTracking(true);
 }
 
+void QFitsView2D::plotSpecSlot() {
+#ifdef HAS_PLOTTING
+    fitsMainWindow->plplotWindow->addBufferData(QString(fitsMainWindow->main_view->getCurrentBufferIndex().c_str()), fitsMainWindow->spectrum->getCenX(), fitsMainWindow->spectrum->getCenY());
+#endif /* HAS_PLOTTING */
+}
+
 void QFitsView2D::copyImageBufferSlot() {
     QString cmd = QString(freeBufferName().to_c_str()) + " = ";
 
@@ -722,6 +752,26 @@ void QFitsView2D::radialProfileSlot() {
     dpuser_widget->executeCommand(cmd);
 }
 
+void QFitsView2D::plotHorizontalSlot() {
+#ifdef HAS_PLOTTING
+    double  imageX = 0,
+            imageY = 0;
+
+    widgetToImageNoSignal(mouseX, mouseY, &imageX, &imageY);
+    fitsMainWindow->plplotWindow->addBufferData(QString(getBufferIndex().c_str()), -1, (int)imageY);
+#endif /* HAS_PLOTTING */
+}
+
+void QFitsView2D::plotVerticalSlot() {
+#ifdef HAS_PLOTTING
+    double  imageX = 0,
+            imageY = 0;
+
+    widgetToImageNoSignal(mouseX, mouseY, &imageX, &imageY);
+    fitsMainWindow->plplotWindow->addBufferData(QString(getBufferIndex().c_str()), (int)imageX, -1);
+#endif /* HAS_PLOTTING */
+}
+
 void QFitsView2D::lockPosSlot() {
     lockUnlockSpectrum(mousePos);
 }
@@ -894,7 +944,7 @@ void QFitsView2D::mouseMoveEvent(QMouseEvent *m) {
                     }
                     if (distance * fabs(sin(angle)) < 8) {
                         QPixmap pm(cursor_rotate);
-                        setCursor(pm);
+                        setCursor(QCursor(pm));
 //@                    } else if (drawMode == longslitMarker && distance * fabs(sin(angle)) < 16) {
 //@                        setCursor(Qt::SizeAllCursor);
                     } else {
@@ -966,7 +1016,7 @@ void QFitsView2D::mousePressEvent(QMouseEvent *m) {
             if (fitsMainWindow->main_view->getCurrentBuffer() != getMyBuffer())
                 fitsMainWindow->main_view->setCurrentBufferFromWidget(getMyBuffer());
         }
-    } else if (m->button() == Qt::MidButton) {
+    } else if (m->button() == Qt::MiddleButton) {
         double  imageX = 0,
                 imageY = 0;
 
@@ -989,19 +1039,33 @@ void QFitsView2D::mousePressEvent(QMouseEvent *m) {
             exportSpecAsciiAct->setVisible(false);
             exportSpecFitsAct->setVisible(false);
             copySpecBufferAct->setVisible(false);
+#ifdef HAS_PLOTTING
+            plotSpecAct->setVisible(false);
+#endif /* HAS_PLOTTING */
             copyImageBufferAct->setVisible(false);
             saveImageFitsAct->setVisible(false);
             lockPosAct->setVisible(false);
             useStdStarAct->setVisible(false);
+#ifdef HAS_PLOTTING
+            plotHorizontalAct->setVisible(true);
+            plotVerticalAct->setVisible(true);
+#endif /* HAS_PLOTTING */
         } else {
             // we have a cube
             exportSpecAsciiAct->setVisible(true);
             exportSpecFitsAct->setVisible(true);
             copySpecBufferAct->setVisible(true);
+#ifdef HAS_PLOTTING
+            plotSpecAct->setVisible(true);
+#endif /* HAS_PLOTTING */
             copyImageBufferAct->setVisible(true);
             saveImageFitsAct->setVisible(true);
             lockPosAct->setVisible(true);
             useStdStarAct->setVisible(true);
+#ifdef HAS_PLOTTING
+            plotHorizontalAct->setVisible(false);
+            plotVerticalAct->setVisible(false);
+#endif /* HAS_PLOTTING */
         }
 
         if (sourceContinuumMarkers->hasAnyMarkers() ||
@@ -1114,12 +1178,15 @@ void QFitsView2D::keyPressEvent( QKeyEvent *e ) {
 //                    activateWindow();
                     break;
                 case Qt::Key_D:
+                case Qt::Key_P:
                     if ((mousemode == None) && !locked) {
                         initialpos = mapFromGlobal(pos);
                         currentpos = mapFromGlobal(pos);
                         setCursor(Qt::CrossCursor);
                         mousemode = drawDistMarker;
                     }
+                    if (e->key() == Qt::Key_P) distancemode = Pixels;
+                    else distancemode = Arcsecs;
                     break;
                 case Qt::Key_L:
                     lockUnlockSpectrum(pos);
@@ -1212,6 +1279,7 @@ void QFitsView2D::keyReleaseEvent(QKeyEvent *e) {
 
     switch (e->key()) {
         case Qt::Key_D:
+        case Qt::Key_P:
             if (!e->isAutoRepeat() && !locked) {
                 mousemode = None;
                 update(visibleRegion());
@@ -1490,8 +1558,13 @@ void QFitsView2D::drawDistanceMarker(QPainter &p) {
                          sbFits->crtype,
                          &xpos, &ypos);
 
+                inPixels = (x == xpos && y == ypos);
                 ixpos = x;
                 iypos = y;
+                dix = -x;
+                diy = -y;
+/*                ixpos = xpos;
+                iypos = ypos; */
                 widgetToImageNoSignal(currentpos.x(), currentpos.y(), &x, &y);
                 x = (int)x;
                 y = (int)y;
@@ -1511,8 +1584,12 @@ void QFitsView2D::drawDistanceMarker(QPainter &p) {
                          sbFits->crtype,
                          &xpos, &ypos);
 
-                dx = (xpos - ixpos) * 3600.;
-                dy = (ypos - iypos) * 3600.;
+                inPixels = (inPixels && x == xpos && y == ypos);
+
+/*                dx = (xpos - ixpos) * 3600.;
+                dy = (ypos - iypos) * 3600.; */
+                dix += x;
+                diy += y;
 
                 xpos = x;
                 ypos = y;
@@ -1520,18 +1597,18 @@ void QFitsView2D::drawDistanceMarker(QPainter &p) {
                 dp_debug("DRAW MARKER xpos: %g, ypos: %g", xpos, ypos);
                 dp_debug("DRAW MARKER ixpos: %g, iypos: %g", ixpos, iypos);
 
-                dix = (int)(xpos - ixpos);
+/*                dix = (int)(xpos - ixpos);
                 diy = (int)(ypos - iypos);
 
-                if (sbFits->hasRefPix()) {
+                if (sbFits->hasRefPix()) { */
                     dx = (xpos - ixpos) * sbFits->getCDELT(1) * 3600.;
                     dy = (ypos - iypos) * sbFits->getCDELT(2) * 3600.;
-                } else {
+/*                } else {
                     dx = dix;
                     dy = diy;
                     inPixels = true;
                 }
-
+*/
                 if ((sb->getRotation() == 90) || (sb->getRotation() == 270)) {
                     std::swap(dx, dy);
                     std::swap(dix, diy);
@@ -1561,14 +1638,17 @@ void QFitsView2D::drawDistanceMarker(QPainter &p) {
             p.setPen(QColor(255, 255, 255));
             p.setBrush(QColor(0, 0, 0));
             p.drawLine(initialpos, currentpos);
-            QString text = QString::number(sqrt(dx*dx+dy*dy), 'g', 4);
-            if (!inPixels) {
+            QString text;
+            if (inPixels || distancemode == Pixels) {
+                text = QString::number(sqrt(dix*dix+diy*diy), 'g', 4);
+            } else {
+                text = QString::number(sqrt(dx*dx+dy*dy), 'g', 4);
                 text += "''"; // unit is arcsec
             }
 
             QFontMetrics metrics(font());
-            int w = metrics.width(text),
-                h = metrics.height();
+            int w = metrics.boundingRect(text).width(),
+                h = metrics.boundingRect(text).height();
             QPoint point = (currentpos + initialpos) / 2;
             if (currentpos.x() > initialpos.x()) {
                 point.setX(point.x() - w);
@@ -1578,16 +1658,18 @@ void QFitsView2D::drawDistanceMarker(QPainter &p) {
             }
 
             p.setPen(QColor(0, 0, 0));
-            p.drawRoundRect(point.x()-2, point.y()+2, w+4, -h, 50, 99);
+            p.drawRoundedRect(point.x()-2, point.y()+2, w+4, -h, 50, 99);
             p.setPen(QColor(255, 255, 255));
             p.drawText(point, text);
 
             p.drawLine(currentpos.x(), initialpos.y(), currentpos.x(), currentpos.y());
-            text = QString::number(dy, 'g', 4);
-            if (!inPixels) {
+            if (inPixels || distancemode == Pixels) {
+                text = QString::number(diy, 'g', 4);
+            } else {
+                text = QString::number(dy, 'g', 4);
                 text += "''"; // unit is arcsec
             }
-            w = metrics.width(text);
+            w = metrics.boundingRect(text).width();
             if (currentpos.x() > initialpos.x()) {
                 point.setX(currentpos.x() + 2);
             } else {
@@ -1595,16 +1677,18 @@ void QFitsView2D::drawDistanceMarker(QPainter &p) {
             }
 
             p.setPen(QColor(0, 0, 0));
-            p.drawRoundRect(point.x()-2, point.y()+2, w+4, -h, 50, 99);
+            p.drawRoundedRect(point.x()-2, point.y()+2, w+4, -h, 50, 99);
             p.setPen(QColor(255, 255, 255));
             p.drawText(point, text);
 
             p.drawLine(initialpos.x(), initialpos.y(), currentpos.x(), initialpos.y());
-            text = QString::number(dx, 'g', 4);
-            if (!inPixels) {
+            if (inPixels || distancemode == Pixels) {
+                text = QString::number(dix, 'g', 4);
+            } else {
+                text = QString::number(dx, 'g', 4);
                 text += "''"; // unit is arcsec
             }
-            w = metrics.width(text);
+            w = metrics.boundingRect(text).width();
             point.setX((currentpos.x() + initialpos.x() - w) / 2);
             if (currentpos.y() > initialpos.y()) {
                 point.setY(initialpos.y() - 2);
@@ -1613,7 +1697,7 @@ void QFitsView2D::drawDistanceMarker(QPainter &p) {
             }
 
             p.setPen(QColor(0, 0, 0));
-            p.drawRoundRect(point.x()-2, point.y()+2, w+4, -h, 50, 99);
+            p.drawRoundedRect(point.x()-2, point.y()+2, w+4, -h, 50, 99);
             p.setPen(QColor(255, 255, 255));
             p.drawText(point, text);
         }
