@@ -4,18 +4,17 @@
 #include "QFitsMainWindow.h"
 #include "QFitsMainView.h"
 #include "QFitsWidget1D.h"
+#include "QFitsCubeSpectrum.h"
 #include "QFitsSingleBuffer.h"
 #include "QFitsGlobal.h"
 #include "QFitsToolBar.h"
 #include "fitting.h"
 #include "qtdpuser.h"
 
-#include "resources/xicon_red.xpm"
-
 QFitsWidget1D::QFitsWidget1D(QFitsBaseBuffer *bb) : QFitsBaseWidget(bb)
 {
     QPalette pal(palette());
-    pal.setColor(QPalette::Background, Qt::white);
+    pal.setColor(QPalette::Window, Qt::white);
 
 //    setAutoFillBackground(true);
     setPalette(pal);
@@ -32,7 +31,10 @@ QFitsWidget1D::QFitsWidget1D(QFitsBaseBuffer *bb) : QFitsBaseWidget(bb)
     params.create(10, 2, R8);
     fitparams.create(1, 1, R8);
     plotter = new QFitsView1D(this);
-
+    rangeControl = new QFitsSpectrumRangeControl(this);
+    connect(rangeControl, SIGNAL(channelRangeChanged()),
+            this, SLOT(updateRange()));
+    connect(plotter, SIGNAL(rangeChanged(const double &,const double &)), this, SLOT(rangeChanged(const double &,const double &)));
     fitresults = new QTableWidget(0, 4, this);
     QStringList headers;
     headers.append("Flux");
@@ -71,11 +73,13 @@ void QFitsWidget1D::resizeEvent(QResizeEvent *e) {
         plotter->setGeometry(fitresults->width(),
                              0,
                              width() - fitresults->width(),
-                             height());
+                             height() - rangeControl->height());
         fitresults->resize(fitresults->width(), height());
     } else {
-        plotter->setGeometry(0, 0, width(), height());
+        plotter->setGeometry(0, 0, width(), height() - rangeControl->height());
     }
+    rangeControl->setGeometry(0, size().height()-rangeControl->height(),
+                                   size().width(), rangeControl->height());
     QFitsBaseWidget::resizeEvent(e);
 }
 
@@ -83,15 +87,18 @@ void QFitsWidget1D::paintEvent(QPaintEvent *e) {
     if ((fitparams.Nelements() > 1) || (fitx[0] != 0. && fitx[1] != 0)) {
         Fits    xvalues,
                 yvalues;
-        int x1 = (int)(plotter->getCrpix() + (fitx[0] - plotter->getCrval()) /
-                                                                    plotter->getCdelt() + 0.5),
-            x2 = (int)(plotter->getCrpix() + (fitx[1] - plotter->getCrval()) /
-                                                                    plotter->getCdelt() + 0.5);
+//        int x1 = (int)(plotter->getCrpix() + (fitx[0] - plotter->getCrval()) /
+//                                                                    plotter->getCdelt() + 0.5),
+//            x2 = (int)(plotter->getCrpix() + (fitx[1] - plotter->getCrval()) /
+//                                                                    plotter->getCdelt() + 0.5);
+        int x1 = wavepix(fitx[0], plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave()),
+            x2 = wavepix(fitx[1], plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
         xvalues.create(labs(x2 - x1) + 1, 1, R8);
         for (int i = 0; i < xvalues.Nelements(); i++) {
-            xvalues.r8data[i] = plotter->getCrval() +
-                                ((double)(i+MIN(x1,x2)) - plotter->getCrpix()) *
-                                                                 plotter->getCdelt();
+//            xvalues.r8data[i] = plotter->getCrval() +
+//                                ((double)(i+MIN(x1,x2)) - plotter->getCrpix()) *
+//                                                                 plotter->getCdelt();
+            xvalues.r8data[i] = pixwave(i+MIN(x1,x2), plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
         }
 
         if (fitestimates->getShowEstimates()) {
@@ -129,10 +136,12 @@ void QFitsWidget1D::keyReleaseEvent(QKeyEvent *e) {
             double xpos2 = plotter->pixelToWavelength(
                                           bend - plotter->getMargin() - plotter->getFw());
 
-            int start = (int)(plotter->getCrpix() + (xpos1 - plotter->getCrval()) /
-                                                          plotter->getCdelt() + 0.5);
-            int end = (int)(plotter->getCrpix() + (xpos2 - plotter->getCrval()) /
-                                                          plotter->getCdelt() + 0.5);
+//            int start = (int)(plotter->getCrpix() + (xpos1 - plotter->getCrval()) /
+//                                                          plotter->getCdelt() + 0.5);
+//            int end = (int)(plotter->getCrpix() + (xpos2 - plotter->getCrval()) /
+//                                                          plotter->getCdelt() + 0.5);
+            int start = wavepix(xpos1, plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
+            int end = wavepix(xpos2, plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
             if (!getMyBuffer()->isMultiBufferChild()) {
                 dpuser_widget->executeCommand(
                         QString(getBufferIndex().c_str()) +
@@ -168,6 +177,14 @@ void QFitsWidget1D::updateScaling() {
     plotter->updateScaling();
 }
 
+void QFitsWidget1D::updateRange() {
+    updateScaling();
+}
+
+void QFitsWidget1D::rangeChanged(const double &min, const double &max) {
+    rangeControl->setPhysicalRange(min, max);
+}
+
 void QFitsWidget1D::setXRange(const double &min, const double &max) {
     plotter->setXRange(min, max);
 }
@@ -178,6 +195,7 @@ void QFitsWidget1D::setYRange(const double &min, const double &max) {
 
 void QFitsWidget1D::setData() {
     plotter->setData();
+    rangeControl->setSpectrum(plotter->getNdata(), plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
 }
 
 void QFitsWidget1D::hide() {
@@ -348,14 +366,14 @@ void QFitsWidget1D::plotterMouseMoveEvent(QMouseEvent *e) {
         return;
     }
 
-    if (!isActiveWindow()) activateWindow();
+//    if (!isActiveWindow()) activateWindow();
 
     double value = 0.0;
     double xpos = plotter->pixelToWavelength(
                                   e->pos().x() - plotter->getMargin() - plotter->getFw());
 
-    int ppos = (int)(plotter->getCrpix() + (xpos - plotter->getCrval()) /
-                                                          plotter->getCdelt() + 0.5);
+//    int ppos = (int)(plotter->getCrpix() + (xpos - plotter->getCrval()) / plotter->getCdelt() + 0.5);
+    int ppos = wavepix(xpos, plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
     if (ppos > 0 && ppos <= plotter->getNdata()) {
         value = plotter->getYdata()[ppos-1];
     }
@@ -450,14 +468,18 @@ void QFitsWidget1D::dofit() {
     Fits xvalues, yvalues, errors;
 
     double s = (fity[1] - fity[0]) / (fitx[1] - fitx[0]);
-    x1 = (int)(plotter->getCrpix() + (fitx[0] - plotter->getCrval()) / plotter->getCdelt() + 0.5);
-    x2 = (int)(plotter->getCrpix() + (fitx[1] - plotter->getCrval()) / plotter->getCdelt() + 0.5);
+//    x1 = (int)(plotter->getCrpix() + (fitx[0] - plotter->getCrval()) / plotter->getCdelt() + 0.5);
+//    x2 = (int)(plotter->getCrpix() + (fitx[1] - plotter->getCrval()) / plotter->getCdelt() + 0.5);
+    x1 = wavepix(fitx[0], plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
+    x2 = wavepix(fitx[1], plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
+
     xvalues.create(labs(x2 - x1) + 1, 1, R8);
     yvalues.create(labs(x2 - x1) + 1, 1, R8);
     errors.create(labs(x2 - x1) + 1, 1, R8);
     for (i = 0; i < xvalues.Nelements(); i++) {
-        xvalues.r8data[i] =
-            plotter->getCrval() + ((double)(i+1+MIN(x1,x2)) - plotter->getCrpix()) * plotter->getCdelt();
+//        xvalues.r8data[i] =
+//            plotter->getCrval() + ((double)(i+1+MIN(x1,x2)) - plotter->getCrpix()) * plotter->getCdelt();
+        xvalues.r8data[i] = pixwave(i+1+MIN(x1,x2), plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
         yvalues.r8data[i] = plotter->getYdata()[i+MIN(x1,x2)];
         errors.r8data[i] = 1.0;
     }
@@ -474,7 +496,11 @@ void QFitsWidget1D::dofit() {
     double min, max, xscale, xoffset, yoffset, yscale;
     bool good;
 
-    // rescale yvalues to lie roughly between -1 and 1
+// we fit twice. First using the initial estimates to a rescaled version of the data
+// and all errors set to one. Then we subtract the fit, estimate the error from the rms
+// of the residual, and then fit again to the original data
+
+// rescale yvalues to lie roughly between -1 and 1
     yoffset = yvalues.get_median();
     yvalues -= yoffset;
     yvalues.get_minmax(&min, &max);
@@ -505,12 +531,16 @@ void QFitsWidget1D::dofit() {
 
     good = multifuncfit(fitparams, &chisq, xvalues, yvalues, errors, types);
 
+// rescale back
     fitparams.r8data[1] /= xscale;
     fitparams.r8data[0] = fitparams.r8data[0] - fitparams.r8data[1] * xoffset;
 
     fitparams.r8data[0] = fitparams[0] * yscale + yoffset;
     fitparams.r8data[1] *= yscale;
 
+    fitparams.r8data[fitparams.C_I(0, 1)] =
+                       (fitparams.r8data[fitparams.C_I(0, 1)]) * yoffset;
+
     for (g = 0; g < ngauss; g++) {
         fitparams.r8data[2+g*3] = (fitparams.r8data[2+g*3]) * yscale;
         fitparams.r8data[fitparams.C_I(2+g*3, 1)] =
@@ -525,6 +555,21 @@ void QFitsWidget1D::dofit() {
         fitparams.r8data[fitparams.C_I(4+g*3, 1)] =
                            (fitparams.r8data[fitparams.C_I(4+g*3, 1)]) * xscale;
     }
+
+// now prepare for the second fit
+    for (i = 0; i < xvalues.Nelements(); i++) {
+        xvalues.r8data[i] = pixwave(i+1+MIN(x1,x2), plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
+        yvalues.r8data[i] = plotter->getYdata()[i+MIN(x1,x2)];
+    }
+    Fits evalues;
+    evaluate_multifunc(evalues, xvalues, params, types);
+    evalues -= yvalues;
+    double serror = evalues.get_meddev();
+    for (i = 0; i < errors.Nelements(); i++) {
+        errors.r8data[i] = serror;
+    }
+    good = multifuncfit(fitparams, &chisq, xvalues, yvalues, errors, types);
+
     updateTable(fitparams);
     update();
 }
@@ -549,12 +594,15 @@ void QFitsWidget1D::subtractfit() {
         int i, x1, x2, g;
         Fits xvalues, yvalues, gparams;
 
-        x1 = (int)(plotter->getCrpix() + (fitx[0] - plotter->getCrval()) / plotter->getCdelt() + 0.5);
-        x2 = (int)(plotter->getCrpix() + (fitx[1] - plotter->getCrval()) / plotter->getCdelt() + 0.5);
+//        x1 = (int)(plotter->getCrpix() + (fitx[0] - plotter->getCrval()) / plotter->getCdelt() + 0.5);
+//        x2 = (int)(plotter->getCrpix() + (fitx[1] - plotter->getCrval()) / plotter->getCdelt() + 0.5);
+        x1 = wavepix(fitx[0], plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
+        x2 = wavepix(fitx[1], plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
         xvalues.create(labs(x2 - x1) + 1, 1, R8);
         for (i = 0; i < xvalues.Nelements(); i++) {
-            xvalues.r8data[i] =
-                plotter->getCrval() + ((double)(i+1+MIN(x1,x2)) - plotter->getCrpix()) * plotter->getCdelt();
+//            xvalues.r8data[i] =
+//                plotter->getCrval() + ((double)(i+1+MIN(x1,x2)) - plotter->getCrpix()) * plotter->getCdelt();
+            xvalues.r8data[i] = pixwave(i+1+MIN(x1,x2), plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
         }
         gparams.create(4, 1, R8);
         gparams.r8data[0] = 0.0;
@@ -586,14 +634,17 @@ void QFitsWidget1D::addfit() {
         int i, x1, x2, g;
         Fits xvalues, yvalues, gparams;
 
-        x1 = (int)(plotter->getCrpix() + (fitx[0] - plotter->getCrval()) /
-                                                          plotter->getCdelt() + 0.5);
-        x2 = (int)(plotter->getCrpix() + (fitx[1] - plotter->getCrval()) /
-                                                          plotter->getCdelt() + 0.5);
+//        x1 = (int)(plotter->getCrpix() + (fitx[0] - plotter->getCrval()) /
+//                                                          plotter->getCdelt() + 0.5);
+//        x2 = (int)(plotter->getCrpix() + (fitx[1] - plotter->getCrval()) /
+//                                                          plotter->getCdelt() + 0.5);
+        x1 = wavepix(fitx[0], plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
+        x2 = wavepix(fitx[1], plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
         xvalues.create(labs(x2 - x1) + 1, 1, R8);
         for (i = 0; i < xvalues.Nelements(); i++) {
-            xvalues.r8data[i] = plotter->getCrval() +
-                                ((double)(i+1+MIN(x1,x2)) - plotter->getCrpix()) * plotter->getCdelt();
+//            xvalues.r8data[i] = plotter->getCrval() +
+//                                ((double)(i+1+MIN(x1,x2)) - plotter->getCrpix()) * plotter->getCdelt();
+            xvalues.r8data[i] = pixwave(i+1+MIN(x1,x2), plotter->getCrpix(), plotter->getCrval(), plotter->getCdelt(), plotter->isLogwave());
         }
         gparams.create(4, 1, R8);
         gparams.r8data[0] = 0.0;
@@ -776,10 +827,11 @@ QFitsTransparentLabel::QFitsTransparentLabel(QFitsWidget1D* parent, QFitsView1D
     setAutoFillBackground(true);
 
     QPalette p;
-    p.setColor(QPalette::Background, QColor(0, 0, 0, 150));
+    p.setColor(QPalette::Window, QColor(0, 0, 0, 150));
     p.setColor(QPalette::WindowText, QColor(255, 255, 255));
     setPalette(p);
     setMouseTracking(true);
+    setAttribute(Qt::WA_ShowWithoutActivating);
 
     theCaption = new QFitsTransparentCaption(this);
     theCaption->setAlignment(Qt::AlignCenter);
@@ -787,7 +839,7 @@ QFitsTransparentLabel::QFitsTransparentLabel(QFitsWidget1D* parent, QFitsView1D
     theCaption->adjustSize();
     theCaption->setGeometry(0, 0, width(), theCaption->height());
 
-    closeButton = new QFitsSimplestButton(QPixmap(xicon_red), theCaption);
+    closeButton = new QFitsSimplestButton(QIcon(":/xicon_red.svg").pixmap(10, 10), theCaption);
     connect(closeButton, SIGNAL(clicked()),
             myParent, SLOT(handleescape()));
 
@@ -849,8 +901,11 @@ QFitsTransparentLabel::QFitsTransparentLabel(QFitsWidget1D* parent, QFitsView1D
     l5 = new QLabelWithoutMouse(viewArea);
     l5->setAlignment(Qt::AlignHCenter);
     QFontMetrics m(l1->font());
-    l1->setMinimumSize(m.width("Lorentz:"), 10);
-    int w = m.width("-0.00000e-00");
+// TODO QT6
+//    l1->setMinimumSize(m.horizontalAdvance("Lorentz:"), 10);
+//    int w = m.horizontalAdvance("-0.00000e-00");
+    l1->setMinimumSize(m.boundingRect("Lorentz:").width(), 10);
+    int w = m.boundingRect("-0.00000e-00").width();
     l2->setMinimumSize(w, 10);
     l3->setMinimumSize(w, 10);
     l4->setMinimumSize(w, 10);
@@ -868,7 +923,7 @@ QFitsTransparentLabel::QFitsTransparentLabel(QFitsWidget1D* parent, QFitsView1D
 
     marker = new QLabel(viewArea);
     marker->setAutoFillBackground(true);
-    p.setColor(QPalette::Background, QColor(0, 0, 255, 100));
+    p.setColor(QPalette::Window, QColor(0, 0, 255, 100));
     marker->setPalette(p);
     marker->stackUnder(l1);
 
@@ -876,7 +931,7 @@ QFitsTransparentLabel::QFitsTransparentLabel(QFitsWidget1D* parent, QFitsView1D
     marker2->setAutoFillBackground(true);
     marker2->setGeometry(0, 0, 100, 30);
     marker2->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
-    p.setColor(QPalette::Background, QColor(255, 0, 0, 100));
+    p.setColor(QPalette::Window, QColor(255, 0, 0, 100));
     marker2->setPalette(p);
     marker2->hide();
 
@@ -1030,11 +1085,11 @@ void QFitsTransparentLabel::showValues() {
         hide();
     } else {
         QPalette p;
-        p.setColor(QPalette::Background, QColor(0, 0, 255, 100));
+        p.setColor(QPalette::Window, QColor(0, 0, 255, 100));
         p.setColor(QPalette::WindowText, QColor(255, 255, 255));
         if (showEstimates) fitestimatesTab->setPalette(p);
         else fitresultsTab->setPalette(p);
-        p.setColor(QPalette::Background, QColor(50, 50, 50, 150));
+        p.setColor(QPalette::Window, QColor(50, 50, 50, 150));
         p.setColor(QPalette::WindowText, QColor(150, 150, 150));
         if (showEstimates) fitresultsTab->setPalette(p);
         else fitestimatesTab->setPalette(p);
@@ -1202,7 +1257,7 @@ QFitsTransparentCaption::QFitsTransparentCaption(QFitsTransparentLabel *parent)
 {
     setAutoFillBackground(true);
     QPalette p;
-    p.setColor(QPalette::Background, QColor(0, 0, 200, 150));
+    p.setColor(QPalette::Window, QColor(0, 0, 200, 150));
     p.setColor(QPalette::WindowText, QColor(255, 255, 255));
     setPalette(p);
     setCursor(Qt::ArrowCursor);
