@@ -1,5 +1,6 @@
 #include <QMessageBox>
 #include <QPainter>
+#include <QPainterPath>
 #include <QMouseEvent>
 #include <QScrollBar>
 
@@ -14,8 +15,10 @@
 #include "fits.h"
 #include "qtdpuser.h"
 
+extern "C" {
 #include "resources/action_stop.xpm"
 #include "resources/icon_accept.xpm"
+}
 
 void Contour(Fits &d, Fits &z, QPainter &p);
 void dpPGCONX(Fits *d, Fits &z, QPainter &p);
@@ -97,8 +100,9 @@ void dpContour::paintEvent(QPaintEvent *e) {
 //    QReadLocker locker(&buffersLock);
 
     if (myParent->manualLevels) {
-        QStringList l = myParent->levels->text().split(",", QString::SkipEmptyParts);
-
+// TODO QT6
+//        QStringList l = myParent->levels->text().split(",", Qt::SkipEmptyParts);
+				QStringList l = myParent->levels->text().split(",", QString::SkipEmptyParts);
         levels->create(l.size(), 1, R8);
         for (int i = 0; i < l.size(); i++) {
             levels->r8data[i] = l[i].toDouble();
