@@ -82,6 +82,10 @@ void QFitsWidgetTable::init(dpuserType *dpt) {
     tableModel->setDpData(dpt);
 }
 
+void QFitsWidgetTable::copyImage(int) {
+    tableView->copySelectionToClipboard();
+}
+
 void QFitsWidgetTable::resizeColumns() {
     // these commands are very slow... implement manually should get faster.
     // (e.g. search max/min value, calculate string length, set column width accordingly)
@@ -482,26 +486,26 @@ QVariant QFitsTableModel::dataFromDpuserType(dpuserType *dpt, int row, int col)
             return QVariant();
         }
 
-        QVariant tmp;
+        QString tmp;
         FitsBitpix bitpix = fits->getType();
         switch(bitpix) {
         case C16: //complex
-            tmp = FormatComplexQString(dpComplex(fits->cdata[i].r, fits->cdata[i].i)).c_str();
+            tmp = FormatComplexdpString(dpComplex(fits->cdata[i].r, fits->cdata[i].i)).c_str();
             break;
         case R8: //double
             tmp = QString::number(fits->r8data[i], 'g', 16);
             break;
         case R4: //float
-            tmp = fits->r4data[i];
+            tmp = QString::number(fits->r4data[i]);
             break;
         case I1: //unsigned char
-            tmp = fits->i1data[i] * fits->getBSCALE() + fits->getBZERO();
+            tmp = QString::number(fits->i1data[i] * fits->getBSCALE() + fits->getBZERO());
             break;
         case I2: //short
-            tmp = fits->i2data[i] * fits->getBSCALE() + fits->getBZERO();
+            tmp = QString::number(fits->i2data[i] * fits->getBSCALE() + fits->getBZERO());
             break;
         case I4: //int
-            tmp = fits->i4data[i] * fits->getBSCALE() + fits->getBZERO();
+            tmp = QString::number(fits->i4data[i] * fits->getBSCALE() + fits->getBZERO());
             break;
         case I8: //long long
             tmp = QString::number(fits->i8data[i] * fits->getBSCALE() + fits->getBZERO(), 'g', 25);
@@ -509,7 +513,7 @@ QVariant QFitsTableModel::dataFromDpuserType(dpuserType *dpt, int row, int col)
         default:
             break;
         }
-        return tmp;
+        return QVariant(' ' + tmp); // TODO: 12 Feb 2021: Prepending a space is a hack so numbers are shown sensibly when table cell is not wide enough to show full number (BUG in Qt as of at least 5.14?)
     } else if (dpt->type == typeStrarr) {
         if (row < dpt->arrvalue->size()) {
             QVariant tmp = dpt->arrvalue->at(row).to_c_str();
@@ -1184,6 +1188,64 @@ void QFitsTableView::copySelectionToNewBuffer() {
     }
 }
 
+void QFitsTableView::copySelectionToClipboard() {
+    QModelIndexList selected = selectionModel()->selectedIndexes();
+    if (selected.isEmpty()) return;
+    int n = selected.size();
+    if (selected.size() > 256*256) {
+        QMessageBox::warning(this, "QFitsView", "Too many data points to be copied");
+        return;
+    }
+
+    QMimeData *copyData = new QMimeData();
+    QStringList entries;
+    QString line;
+
+    QFitsTableModel *tm = myParent->getTableModel();
+    if (dynamic_cast<QFitsTableModelMulti*>(tm) == NULL) {
+        // dpuserType
+        int row = selected[0].row();
+        for (int i = 0; i < selected.size(); i++) {
+            QVariant tmp = tm->data(selected[i]);
+            if (selected[i].row() != row) {
+                line.remove(line.length()-1, 1);
+                entries << line;
+                line.clear();
+                row = selected[i].row();
+            }
+            line += tmp.toString();
+            line += ",";
+        }
+        line.remove(line.length()-1, 1);
+        entries << line;
+        line.clear();
+    } else {
+        // dpuserTypeList
+        QFitsTableModelMulti *tmm = dynamic_cast<QFitsTableModelMulti*>(myParent->getTableModel());
+        if (tmm != NULL) {
+            int row = selected[0].row();
+            for (int i = 0; i < selected.size(); i++) {
+                QVariant tmp = tmm->data(selected[i]);
+                if (selected[i].row() != row) {
+                    line.remove(line.length()-1, 1);
+                    entries << line;
+                    line.clear();
+                    row = selected[i].row();
+                }
+                line += tmp.toString();
+                line += ",";
+            }
+            line.remove(line.length()-1, 1);
+            entries << line;
+            line.clear();
+        }
+    }
+    entries << "";
+    copyData->setText(entries.join("\n"));
+    QApplication::clipboard()->setMimeData(copyData);
+}
+
+
 void QFitsTableView::setSpectrumVertical(int) {
     bool locked = (myParent->getBaseBuffer()->getLockedSingleBuffer() != NULL);
     if (locked) {
@@ -1223,9 +1285,10 @@ void QFitsTableView::plotSpectrumVertical() {
                     brute_rangeVertical.first  = 1;
                     brute_rangeVertical.second = plotSpectrum->Naxis(1);
                 }
-
-                myParent->getBaseBuffer()->mtv_setMinMaxXbrute(brute_rangeVertical.first,
-                                                               brute_rangeVertical.second);
+                myParent->getBaseBuffer()->setSpecChannelMinX(brute_rangeVertical.first);
+                myParent->getBaseBuffer()->setSpecChannelMaxX(brute_rangeVertical.second);
+//                myParent->getBaseBuffer()->mtv_setMinMaxXbrute(brute_rangeVertical.first,
+//                                                               brute_rangeVertical.second);
                 setupCubeSpectrum();
                 calcScrollContinuum();
             }
@@ -1278,6 +1341,7 @@ void QFitsTableView::plotSpectrumHorizontal() {
                 brute_rangeHorizontal.first  = 1;
                 brute_rangeHorizontal.second = plotSpectrum->Naxis(1);
             }
+            // TODO: Check if this is really what we want or if, as for the Vertical case, we just want to set the Channel Min/Max
             myParent->getBaseBuffer()->mtv_setMinMaxXbrute(brute_rangeHorizontal.first,
                                     brute_rangeHorizontal.second);
             setupCubeSpectrum();
