@@ -1,7 +1,7 @@
 #include <QMessageBox>
 #include <QPainter>
 #include <QScreen>
-#include <QDesktopWidget>
+//#include <QDesktopWidget>
 #include <QMouseEvent>
 
 #include "QFitsMainWindow.h"
@@ -112,18 +112,18 @@ void QFitsWidgetWiregrid::mouseMoveEvent(QMouseEvent *event) {
         update();
     } else {
         QPoint epos = mapToGlobal(event->pos());
-        QPixmap p = QApplication::screens().at(0)->grabWindow(QApplication::desktop()->winId(), epos.x() - 15, epos.y() - 15, 30, 30);
+        QPixmap p = QApplication::screens().at(0)->grabWindow(0, epos.x() - 15, epos.y() - 15, 30, 30);
         emit updateMagnifier(p);
     }
 }
 
 void QFitsWidgetWiregrid::wheelEvent(QWheelEvent* event) {
-    if (event->delta() > 0) {
+    if (event->angleDelta().y() > 0) {
         // zoom in
-        scale += (float)event->delta()/120;
+        scale += (float)event->angleDelta().y()/120;
     } else {
         // zoom out
-        scale -= -(float)event->delta()/120;
+        scale -= -(float)event->angleDelta().y()/120;
     }
     update();
 }
@@ -219,10 +219,10 @@ void QFitsWidgetWiregrid::paintEvent(QPaintEvent*) {
         p.save();
         p.fillRect(0, 0, width(), height(), QColor(255, 255, 255));
 //        calcMesh(x1, x2, y1, y2, yRot * M_PI / 180., xRot * M_PI / 180.);
-        QMatrix m;
-//        m.setMatrix(1, 0, 0, -1, width() / 2, height() - height() / 2);
-        m.setMatrix(-1, 0, 0, -1, width() / 2, (height()-20) - fabs(xRot) / 90. * ((height()-10) / 2));
-        p.setWorldMatrix(m);
+        QTransform m(-1, 0, 0, -1, width() / 2, (height()-20) - fabs(xRot) / 90. * ((height()-10) / 2));
+//        m.setMatrix(-1, 0, 0, -1, width() / 2, (height()-20) - fabs(xRot) / 90. * ((height()-10) / 2));
+//        m.setMatrix(-1, 0, 0, 0, -1, 0, width() / 2, (height()-20) - fabs(xRot) / 90. * ((height()-10) / 2), 0);
+        p.setWorldTransform(m);
         QPen pen(QColor(0, 0, 0));
         pen.setWidth(0);
         p.setPen(pen);
