@@ -18,7 +18,10 @@
 #include "dpstringlist.h"
 #include "fitting.h"
 #include "astrolib.h"
+
+extern "C" {
 #include "resources/fileopen.xpm"
+}
 
 _userDialogs userDialogs;
 extern Fits *markposBuffer;
@@ -1861,7 +1864,7 @@ imRedLongslit::imRedLongslit(QFitsMainWindow *parent) : imRedGeneric(parent) {
             pen.setWidth(1);
             pen.setColor(QColor(0, 0, 0));
             plotarea->graph(0)->setPen(pen);
-            mainLayout->addWidget(plotarea, 4, 0);
+            mainLayout->addWidget(plotarea, 4);
             addButtons(plotarea);
             setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
         }
