@@ -142,7 +142,7 @@ dpuserType plusNode::evaluate() {
                 break;
             case typeCom:
                 result.clone(leftvalue);
-                *result.svalue += FormatComplexQString(*rightvalue.cvalue);
+                *result.svalue += FormatComplexdpString(*rightvalue.cvalue);
                 break;
             case typeStr:
                 result.clone(leftvalue);
