@@ -62,7 +62,7 @@ void print(dpuserType &v, int options) {
 		case typeCon: dp_output("%li\n", v.lvalue); return;
 		case typeDbl: dp_output("%6g\n", v.dvalue); return;
                 case typeStr: dp_output_string("%s\n", v.svalue->c_str()); return;
-        case typeCom: dp_output("%s\n", FormatComplexQString(*v.cvalue).c_str()); return;
+        case typeCom: dp_output("%s\n", FormatComplexdpString(*v.cvalue).c_str()); return;
 		case typeStrarr: {
 			dpint64 i;
 
@@ -135,7 +135,7 @@ void print(dpuserType &v, int options) {
             if ((int)_tmp->membits == -128) {
                 /* typeCom */
                 for (x = 1; x <= _tmp->Naxis(1); x++) {
-                    dp_output("%s\t", FormatComplexQString(dpComplex(_tmp->cdata[n].r, _tmp->cdata[n].i)).c_str());
+                    dp_output("%s\t", FormatComplexdpString(dpComplex(_tmp->cdata[n].r, _tmp->cdata[n].i)).c_str());
                     n++;
                 }
             } else {
@@ -173,7 +173,7 @@ void mem(void) {
                 break;
             case typeStr: dp_output("%s:\tstring\t= %s\n", var.first.c_str(), var.second.toString().c_str());
                 break;
-            case typeCom: dp_output("%s:\tcomplex\t= %s\n", var.first.c_str(), FormatComplexQString(*var.second.cvalue).c_str());
+            case typeCom: dp_output("%s:\tcomplex\t= %s\n", var.first.c_str(), FormatComplexdpString(*var.second.cvalue).c_str());
                 break;
             case typeFits:
                 if (var.second.fvalue->Naxis(0) == 1) {
@@ -527,9 +527,9 @@ void graymap(Fits *v, float min, float max, long itf) {
 		tr[3] = ymin;
 	} else {
 		xmin = 0.0;
-		xmax = (float)v->Naxis(1) - 1.0;
+        xmax = (float)v->Naxis(1) - 1.0;
 		ymin = 0.0;
-		ymax = (float)v->Naxis(2) - 1.0;
+        ymax = (float)v->Naxis(2) - 1.0;
 	}
 
 	cpgenv(xmin, xmax, ymin, ymax, 1, 0);
@@ -555,7 +555,7 @@ void graymap(Fits *v, float min, float max, long itf) {
     cpgsitf(itf);
 	data1 = floatdata(*v);
 	if (data1)
-		cpggray(data1, v->Naxis(1), v->Naxis(2), 1, v->Naxis(1), 1, v->Naxis(2), max, min, tr);
+        cpggray(data1, v->Naxis(1), v->Naxis(2), 1, v->Naxis(1), 1, v->Naxis(2), max, min, tr);
 	if (data1 && (data1 != v->r4data)) free(data1);
 
 	cpgwedg("R", 1.0, 3.0, max, min, " ");
@@ -938,7 +938,7 @@ void help(char *what) {
                 dp_output("string\t= %s\n", dpuser_vars[what].svalue->c_str());
                 break;
             case typeCom:
-                dp_output("complex\t= %s\n", FormatComplexQString(*dpuser_vars[what].cvalue).c_str());
+                dp_output("complex\t= %s\n", FormatComplexdpString(*dpuser_vars[what].cvalue).c_str());
                 break;
             case typeFits: dp_output("fits\t = ");
                 if (dpuser_vars[what].fvalue->Naxis(0) != 3) {
