@@ -15,6 +15,9 @@
 #include "QFitsGlobal.h"
 #endif /* DPQT */
 
+#pragma GCC push_options
+#pragma GCC optimize ("O0")
+
 std::map<std::string, dpuserType> dpuser_vars;
 std::map<std::string, ASTNode *>userprocedures;
 std::map<std::string, std::vector<std::string> >userprocedure_arguments;
@@ -115,6 +118,9 @@ dpuserType writefitsNode::evaluate() {
         return rv;
     }
     rv.fvalue->WriteFITS(fname.c_str());
+
+    updateQFitsViewTitle(&rv);
+
     return rv;
 }
 
@@ -1529,3 +1535,5 @@ dpuserType whereNode::evaluate() {
     }
     return rv;
 }
+
+#pragma GCC pop_options
