@@ -27,7 +27,7 @@ std::ostream& operator<< (std::ostream &out, const dpuserType &value) {
     switch (value.type) {
     case typeCon: out << value.lvalue; break;
     case typeDbl: out << value.dvalue; break;
-    case typeCom: out << FormatComplexQString(*value.cvalue); break;
+    case typeCom: out << FormatComplexdpString(*value.cvalue); break;
     case typeStr: out << *value.svalue; break;
     case typeFitsFile: out << "Fits file with name " << *value.ffvalue; break;
     case typeDpArr: for (auto el:(*value.dparrvalue)) out << *el;
@@ -564,7 +564,7 @@ dpuserType &dpuserType::operator +=(const dpuserType &arg) {
                 *svalue += *CreateString(arg.dvalue);
                 break;
             case typeCom:
-                *svalue += FormatComplexQString(*arg.cvalue);
+                *svalue += FormatComplexdpString(*arg.cvalue);
                 break;
             case typeStr:
                 *svalue += *arg.svalue;
@@ -1463,16 +1463,28 @@ dpuserType ReadFITSBinTable(const char *fname, dpuserType ext, dpuserType col) {
                 rv.fvalue->setFileName(name.c_str());
                 rv.fvalue->setExtensionNumber(extension);
                 rv.fvalue->setColumnNumber(column);
-                if (!_tmp.GetBintableColumn(column, *rv.fvalue)) {
-                    throw dpuserTypeException("readfitsbintable: Could not read column!");
+                if (_tmp.extensionType == TABLE) {
+                    if (!_tmp.GetAsciitableColumn(column, *rv.fvalue)) {
+                        throw dpuserTypeException("readfitsbintable: Could not read column!");
+                    }
+                } else {
+                    if (!_tmp.GetBintableColumn(column, *rv.fvalue)) {
+                        throw dpuserTypeException("readfitsbintable: Could not read column!");
+                    }
                 }
             } else if (type == typeStrarr) {
                 // column contains characters --> String array
                 rv.type = typeStrarr;
                 rv.arrvalue = CreateStringArray();
                 rv.arrvalue->setFileName(name.c_str());
-                if (!_tmp.GetBintableColumn(column, *rv.arrvalue)) {
-                    throw dpuserTypeException("readfitsbintable: Could not read column!");
+                if (_tmp.extensionType == TABLE) {
+                    if (!_tmp.GetAsciitableColumn(column, *rv.arrvalue)) {
+                        throw dpuserTypeException("readfitsbintable: Could not read column!");
+                    }
+                } else {
+                    if (!_tmp.GetBintableColumn(column, *rv.arrvalue)) {
+                        throw dpuserTypeException("readfitsbintable: Could not read column!");
+                    }
                 }
             }
         } else {
@@ -1483,6 +1495,7 @@ dpuserType ReadFITSBinTable(const char *fname, dpuserType ext, dpuserType col) {
             for (int i = 1; i <= count; i++) {
                 dpuserType *dpt = new dpuserType;
                 dpt->type = _tmp.GetBintableType(i);
+
                 if (dpt->type == typeFits) {
                     // column contains values --> Fits
 
@@ -1499,9 +1512,16 @@ dpuserType ReadFITSBinTable(const char *fname, dpuserType ext, dpuserType col) {
 //                        dpt->fvalue->HeaderLength = _tmp.HeaderLength;
 //                        strcpy(dpt->fvalue->header, _tmp.header);
                     }
-                    if (_tmp.GetBintableColumn(i, *dpt->fvalue)) {
-//                        dpt->fvalue->extensionType = BINTABLE;
-                        rv.dparrvalue->push_back(dpt);
+                    if (_tmp.extensionType == TABLE) {
+                        if (_tmp.GetAsciitableColumn(i, *dpt->fvalue)) {
+                            //                        dpt->fvalue->extensionType = BINTABLE;
+                            rv.dparrvalue->push_back(dpt);
+                        } else {
+                            delete dpt;
+                        }
+                    } else if (_tmp.GetBintableColumn(i, *dpt->fvalue)) {
+    //                        dpt->fvalue->extensionType = BINTABLE;
+                            rv.dparrvalue->push_back(dpt);
                     } else {
                         delete dpt;
                     }
@@ -1511,10 +1531,14 @@ dpuserType ReadFITSBinTable(const char *fname, dpuserType ext, dpuserType col) {
                     dpt->arrvalue = new dpStringList;
                     dpt->arrvalue->setFileName(name.c_str());
 
-                    if (_tmp.GetBintableColumn(i, *dpt->arrvalue)) {
-                        rv.dparrvalue->push_back(dpt);
+                    if (_tmp.extensionType == TABLE) {
+                        if (_tmp.GetAsciitableColumn(i, *dpt->arrvalue)) {
+                            rv.dparrvalue->push_back(dpt);
+                        } else delete dpt;
                     } else {
-                        delete dpt;
+                        if (_tmp.GetBintableColumn(i, *dpt->arrvalue)) {
+                            rv.dparrvalue->push_back(dpt);
+                        } else delete dpt;
                     }
                 } else {
                     delete dpt;
