@@ -21,6 +21,92 @@ long idum;
                                 // throw an exception
 #endif
 
+#ifdef HAS_PLPLOT
+extern "C" {
+void plsabort(void(*handler)(const char *));
+void plsexit(int(*handler)(const char *));
+
+FILE *plLibOpen(const char *fn); // to test if font files are available
+extern char *plplotLibDir;
+}
+
+/* Check if the bare minimum of support files for PLPLOT can be found. If not, create them
+ * in a temporary directory
+ */
+
+#include "plplot_fnt.h"
+void dp_plplot_checkfontfiles() {
+    FILE *f = plLibOpen("plstnd5.fnt");
+    if (f == NULL) {
+// Get tempDir-path. TODO: Once we switch to c++17, use std::filesystem::temp_directory_path()
+        std::string tempPath, fname;
+#ifdef WIN
+        char tempP[MAX_PATH + 1];
+        GetTempPath(MAX_PATH, tempP);
+        tempPath = tempP;
+        if (tempPath.at(tempPath.size() - 1) != '\\') tempPath.append("\\");
+#else
+        tempPath = P_tmpdir;
+        if (tempPath.at(tempPath.size() - 1) != '/') tempPath.append("/");
+#endif /* WIN */
+        plplotLibDir = strdup(tempPath.c_str());
+// try once more
+        FILE *f = plLibOpen("plstnd5.fnt");
+        if (f == NULL) {
+            dp_output("PLPLOT font files not found. Creating them in %s.\n", tempPath.c_str());
+            fname = tempPath;
+            fname.append("plstnd5.fnt");
+            f = fopen(fname.c_str(), "w+b");
+            if (f != NULL) {
+                fwrite(plstnd5_fnt, 1, plstnd5_fnt_len, f);
+                fclose(f);
+            }
+            fname = tempPath;
+            fname.append("plxtnd5.fnt");
+            f = fopen(fname.c_str(), "w+b");
+            if (f != NULL) {
+                fwrite(plxtnd5_fnt, 1, plxtnd5_fnt_len, f);
+                fclose(f);
+            }
+            fname = tempPath;
+            fname.append("cmap0_default.pal");
+            f = fopen(fname.c_str(), "w+b");
+            if (f != NULL) {
+                fwrite(cmap0_default_pal, 1, cmap0_default_pal_len, f);
+                fclose(f);
+            }
+            fname = tempPath;
+            fname.append("cmap1_default.pal");
+            f = fopen(fname.c_str(), "w+b");
+            if (f != NULL) {
+                fwrite(cmap1_default_pal, 1, cmap1_default_pal_len, f);
+                fclose(f);
+            }
+        } else fclose(f);
+    } else fclose(f);
+}
+
+void dp_plplot_aborthandler(const char *msg) {
+    dp_output("PLPLOT encountered an abort signal: %s\n", msg);
+}
+
+int dp_plplot_exithandler(const char *msg) {
+    dp_output("PLPLOT encountered an exit signal: %s\n", msg);
+    return 0;
+}
+
+void init_dpplplot() {
+    dp_plplot_checkfontfiles();
+    plsabort(dp_plplot_aborthandler);
+    plsexit(dp_plplot_exithandler);
+}
+#else
+
+void init_dpplplot() {
+}
+
+#endif /* HAS_PLPLOT */
+
 #ifdef HAS_GDL
 int GDLEventHandler();
 #endif /* HAS_GDL */
@@ -400,7 +486,7 @@ dpString *CreateString(const char *s) {
 }
 
 dpString *CreateString(const dpComplex &c) {
-    dpString *string = new dpString(FormatComplexQString(c));
+    dpString *string = new dpString(FormatComplexdpString(c));
     addToListOfOStrings(string);
 
     return string;
@@ -736,16 +822,18 @@ void DeleteDpList(dpuserTypeList *f) {
     f = NULL;
 }
 
-const dpString &FormatComplexQString(const dpComplex &z) {
+// return a dpString from a dpComplex. format defaults to "*.16g", other use could be "%.2f"
+
+const dpString &FormatComplexdpString(const dpComplex &z, const char *format) {
     dpString re, im;
 
     if (isReal(z)) {
-        re.sprintf("%.16g", real(z));
+        re.sprintf(format, real(z));
     } else if (isImag(z)) {
-        im.sprintf("%.16g", fabs(imag(z)));
+        im.sprintf(format, fabs(imag(z)));
     } else {
-        re.sprintf("%.16g", real(z));
-        im.sprintf("%.16g", fabs(imag(z)));
+        re.sprintf(format, real(z));
+        im.sprintf(format, fabs(imag(z)));
     }
 
     // Strip off trailing 0's
@@ -782,53 +870,6 @@ const dpString &FormatComplexQString(const dpComplex &z) {
     return _qtmpstring;
 }
 
-const dpString &FormatComplexQString2(const dpComplex &z) {
-    dpString re, im;
-
-    if (isReal(z)) {
-        re.sprintf("%.2f", real(z));
-    } else if (isImag(z)) {
-        im.sprintf("%.2f", fabs(imag(z)));
-    } else {
-        re.sprintf("%.2f", real(z));
-        im.sprintf("%.2f", fabs(imag(z)));
-    }
-
-    // Strip off trailing 0's
-    if (re.length() > 0) {
-        while (re.right(1) == "0") {
-            re.truncate(re.length() - 1);
-        }
-        if (re.right(1) == ".") {
-            re.truncate(re.length() - 1);
-        }
-    }
-    if (im.length() > 0) {
-        while (im.right(1) == "0") {
-            im.truncate(im.length() - 1);
-        }
-        if (im.right(1) == ".") {
-            im.truncate(im.length() - 1);
-        }
-        if (im == "1") {
-            im = "i";
-        } else {
-            im += "i";
-        }
-    }
-    _qtmpstring = re;
-    if (im.length() > 0) {
-        if (imag(z) < 0.0) {
-            _qtmpstring += "-";
-        } else if (re.length() > 0) {
-            _qtmpstring += "+";
-        }
-        _qtmpstring += im;
-    }
-    return _qtmpstring;
-}
-
-
 // Comparison function for createStringIndex;
 // This function uses the global variable char *_compareString
 char *_compareString;
@@ -1571,3 +1612,17 @@ int pgWord(char *s) {
 
     return -10000;
 }
+
+void updateQFitsViewTitle(dpuserType *ptr) {
+#ifdef DPQT
+    if (ptr == NULL) return;
+    for (auto var:dpuser_vars) {
+        if (((var.second.type == typeFits) && (var.second.fvalue == ptr->fvalue)) ||
+            ((var.second.type == typeDpArr)  && (var.second.dparrvalue == ptr->dparrvalue))) {
+            dpUpdateVar(var.first);
+            break;
+        }
+    }
+#endif /* DPQT */
+}
+
