@@ -25,6 +25,9 @@
 #undef slots
 #endif
 
+#pragma GCC push_options
+#pragma GCC optimize ("O2")
+
 #include <Python.h>
 #include <numpy/arrayobject.h>
 
@@ -978,6 +981,8 @@ int call_python(std::string func_or_pro, std::vector<dpuserType> *args, std::vec
 	return 1;
 }
 
+#pragma GCC pop_options
+
 #else /* HAS_PYTHON */
 
 int call_python(std::string func_or_pro, std::vector<dpuserType> *args, std::vector<std::string> variables, std::map<std::string, std::pair<std::string, dpuserType> > *keyw, bool isFunction, dpuserType &rv) {
