@@ -790,6 +790,316 @@ int pixpos(double xpos, double ypos, double xref, double yref,
   return(*status);
 }  /* end xypix */
 
+int
+worldpix (double xpos, double ypos,
+          double xref, double yref,
+          double xrefpix, double yrefpix, double xinc, double yinc, double rot,
+          double cd1_1, double cd1_2, double cd2_1, double cd2_2,
+          char *type,
+          double *xpix, double *ypix)
+
+/*-----------------------------------------------------------------------*/
+/* routine to determine accurate pixel coordinates for an RA and Dec     */
+/* returns 0 if successful otherwise:                                    */
+/*  1 = angle too large for projection;                                  */
+/*  2 = bad values                                                       */
+/* does: SIN, TAN, ARC, NCP, GLS or SFL, MER, AIT, STG, CAR, COE projections    */
+/* anything else is linear                                               */
+
+{
+  double dx, dy, ra0, dec0, ra, dec, coss, sins, dt, da, dd, sint;
+  double l, m, geo1, geo2, geo3, sinr, cosr, tx, x, a2, a3, a4;
+  double rthea,gamby2,a,b,c,phi,an,rap,v,tthea,co1,co2,co3,co4,ansq; /* COE */
+  double cond2r=1.745329252e-2, deps=1.0e-5, twopi=6.28318530717959;
+
+  double dc1_1, dc1_2, dc2_1, dc2_2, det;
+
+  bool coorflip = FALSE;
+  char ctypes[12][5] ={"-CAR","-SIN","-TAN","-TNX","-ARC","-NCP", "-GLS","-SFL", "-MER",
+     "-AIT", "-STG", "-COE"};
+  int i;
+/* Structure elements */
+  int itype;
+
+  /* Set local projection parameters */
+  rot = rot * M_PI / 180.;
+  cosr = cos (rot);
+  sinr = sin (rot);
+
+  /* Projection type */
+  itype = 0;  /* no default type */
+  for (i=0;i<12;i++) if (!strncmp(type, ctypes[i], 4)) itype = i;
+
+  /* Nonlinear position */
+  if (itype > 0) {
+    if (coorflip) {
+      dec0 = xref * M_PI / 180.;
+      ra0 = yref * M_PI / 180.;
+      dt = xpos - yref;
+      }
+    else {
+      ra0 = xref * M_PI / 180.;
+      dec0 = yref * M_PI / 180.;
+      dt = xpos - xref;
+      }
+
+    /* 0h wrap-around tests added by D.Wells 10/12/1994: */
+    /* Modified to exclude weird reference pixels by D.Mink 2/3/2004 */
+    if (xrefpix*xinc > 180.0 || xrefpix*xinc < -180.0) {
+    if (dt > 360.0) xpos -= 360.0;
+    if (dt < 0.0) xpos += 360.0;
+    }
+    else {
+    if (dt > 180.0) xpos -= 360.0;
+    if (dt < -180.0) xpos += 360.0;
+    }
+    /* NOTE: changing input argument xpos is OK (call-by-value in C!) */
+
+    ra = xpos * M_PI / 180.;
+    dec = ypos * M_PI / 180.;
+
+    /* Compute direction cosine */
+    coss = cos (dec);
+    sins = sin (dec);
+    l = sin(ra-ra0) * coss;
+    sint = sins * sin(dec0) + coss * cos(dec0) * cos(ra-ra0);
+    }
+  else {
+    l = 0.0;
+    sint = 0.0;
+    sins = 0.0;
+    coss = 0.0;
+    ra = 0.0;
+    dec = 0.0;
+    ra0 = 0.0;
+    dec0 = 0.0;
+    m = 0.0;
+    }
+
+  /* Process by case  */
+  switch (itype) {
+
+    case 0:   /* -CAR Cartesian */
+      l = ra - ra0;
+      m = dec - dec0;
+      break;
+
+    case 1:   /* -SIN sin*/
+    if (sint<0.0) return 1;
+    m = sins * cos(dec0) - coss * sin(dec0) * cos(ra-ra0);
+    break;
+
+    case 3:   /* -TNX tan with polynomial correction */
+//    case WCS_TPV:   /* -TPV tan with polynomial correction */
+//    case WCS_ZPX:   /* -ZPX zpn with polynomial correction */
+    case 2:   /* -TAN tan */
+    if (sint<=0.0) return 1;
+    m = sins * sin(dec0) + coss * cos(dec0) * cos(ra-ra0);
+    l = l / m;
+    m = (sins * cos(dec0) - coss * sin(dec0) * cos(ra-ra0)) / m;
+    break;
+
+    case 4:   /* -ARC Arc*/
+    m = sins * sin(dec0) + coss * cos(dec0) * cos(ra-ra0);
+    if (m<-1.0) m = -1.0;
+    if (m>1.0) m = 1.0;
+    m = acos (m);
+    if (m!=0)
+        m = m / sin(m);
+    else
+        m = 1.0;
+    l = l * m;
+    m = (sins * cos(dec0) - coss * sin(dec0) * cos(ra-ra0)) * m;
+    break;
+
+    case 5:   /* -NCP North celestial pole*/
+    if (dec0==0.0)
+        return 1;  /* can't stand the equator */
+    else
+        m = (cos(dec0) - coss * cos(ra-ra0)) / sin(dec0);
+    break;
+
+    case 6:   /* -GLS global sinusoid */
+    case 7:   /* -SFL Samson-Flamsteed */
+    dt = ra - ra0;
+    if (fabs(dec)>twopi/4.0) return 1;
+    if (fabs(dec0)>twopi/4.0) return 1;
+    m = dec - dec0;
+    l = dt * coss;
+    break;
+
+    case 8:   /* -MER mercator*/
+    dt = yinc * cosr + xinc * sinr;
+    if (dt==0.0) dt = 1.0;
+    dy = degrad (yref/2.0 + 45.0);
+    dx = dy + dt / 2.0 * cond2r;
+    dy = log (tan (dy));
+    dx = log (tan (dx));
+    geo2 = degrad (dt) / (dx - dy);
+    geo3 = geo2 * dy;
+    geo1 = cos (degrad (yref));
+    if (geo1<=0.0) geo1 = 1.0;
+    dt = ra - ra0;
+    l = geo1 * dt;
+    dt = dec / 2.0 + twopi / 8.0;
+    dt = tan (dt);
+    if (dt<deps) return 2;
+    m = geo2 * log (dt) - geo3;
+    break;
+
+    case 9:   /* -AIT Aitoff*/
+    l = 0.0;
+    m = 0.0;
+    da = (ra - ra0) / 2.0;
+    if (fabs(da)>twopi/4.0) return 1;
+    dt = yinc*cosr + xinc*sinr;
+    if (dt==0.0) dt = 1.0;
+    dt = degrad (dt);
+    dy = degrad (yref);
+    dx = sin(dy+dt)/sqrt((1.0+cos(dy+dt))/2.0) -
+         sin(dy)/sqrt((1.0+cos(dy))/2.0);
+    if (dx==0.0) dx = 1.0;
+    geo2 = dt / dx;
+    dt = xinc*cosr - yinc* sinr;
+    if (dt==0.0) dt = 1.0;
+    dt = degrad (dt);
+    dx = 2.0 * cos(dy) * sin(dt/2.0);
+    if (dx==0.0) dx = 1.0;
+    geo1 = dt * sqrt((1.0+cos(dy)*cos(dt/2.0))/2.0) / dx;
+    geo3 = geo2 * sin(dy) / sqrt((1.0+cos(dy))/2.0);
+    dt = sqrt ((1.0 + cos(dec) * cos(da))/2.0);
+    if (fabs(dt)<deps) return 3;
+    l = 2.0 * geo1 * cos(dec) * sin(da) / dt;
+    m = geo2 * sin(dec) / dt - geo3;
+    break;
+
+    case 10:   /* -STG Sterographic*/
+    da = ra - ra0;
+    if (fabs(dec)>twopi/4.0) return 1;
+    dd = 1.0 + sins * sin(dec0) + coss * cos(dec0) * cos(da);
+    if (fabs(dd)<deps) return 1;
+    dd = 2.0 / dd;
+    l = l * dd;
+    m = dd * (sins * cos(dec0) - coss * sin(dec0) * cos(da));
+    break;
+
+    case 11:    /* allan: -COE projection added, AW, ESO*/
+    gamby2 = sin (dec0);
+    tthea = tan (dec0);
+    rthea = 1. / tthea;
+    a = -2. * tthea;
+    b = tthea * tthea;
+    c = tthea / 3.;
+    a2 = a * a;
+    a3 = a2 * a;
+    a4 = a2 * a2;
+    co1 = a/2.;
+    co2 = -0.125 * a2 + b/2.;
+    co3 = -0.25 * a*b + 0.0625 * a3 + c/2.0;
+    co4 = -0.125 * b*b - 0.25 * a*c + 0.1875 * b*a2 - (5.0/128.0)*a4;
+    phi = ra0 - ra;
+    an = phi * gamby2;
+    v = dec - dec0;
+    rap = rthea * (1.0 + v * (co1+v * (co2+v * (co3+v * co4))));
+    ansq = an * an;
+    if (cd1_1 != 0.0)
+        l = rap * an * (1.0 - ansq/6.0) * (cd1_1 / fabs(cd1_1));
+    else
+        l = rap * an * (1.0 - ansq/6.0) * (xinc / fabs(xinc));
+    m = rthea - (rap * (1.0 - ansq/2.0));
+    break;
+
+    }  /* end of itype switch */
+
+  /* Convert back to degrees  */
+  if (itype > 0) {
+    dx = l * 180. / M_PI;
+    dy = m * 180. / M_PI;
+    }
+
+  /* For linear or pixel projection */
+  else {
+    dx = xpos - xref;
+    dy = ypos - yref;
+    }
+
+  if (coorflip) {
+    tx = dx;
+    dx = dy;
+    dy = tx;
+    }
+
+  /* Scale and rotate using CD matrix */
+  if (cd1_1 != 0.0) {
+
+// invert cd matrix
+    det = cd1_1 * cd2_2 - cd1_2 * cd2_1;
+    dc1_1 = cd2_2 / det;
+    dc1_2 = -cd1_2 / det;
+    dc2_1 = -cd2_1 / det;
+    dc2_2 = cd1_1 / det;
+
+    tx = dx * dc1_1 + dy * dc2_1;
+    dy = dx * dc1_2 + dy * dc2_2;
+    dx = tx;
+    }
+
+  /* Scale and rotate using CDELTn and CROTA2 */
+  else {
+
+    /* Correct for rotation */
+    if (rot!=0.0) {
+      tx = dx*cosr + dy*sinr;
+      dy = dy*cosr - dx*sinr;
+      dx = tx;
+      }
+
+    /* Scale using CDELT */
+    if (xinc != 0.)
+      dx = dx / xinc;
+    if (yinc != 0.)
+      dy = dy / yinc;
+    }
+
+  /* Convert to pixels  */
+  *xpix = dx + xrefpix;
+
+  /*
+  if (itype == 0) {
+    if (*xpix > wcs->nxpix) {
+      x = *xpix - (360.0 / xinc);
+      if (x > 0.0) *xpix = x;
+      }
+    else if (*xpix < 0) {
+      x = *xpix + (360.0 / xinc);
+      if (x <= wcs->nxpix) *xpix = x;
+      }
+    }
+   */
+  *ypix = dy + yrefpix;
+
+  return 0;
+}  /* end worldpix */
+
+
+/* Calculate spectral axis value at position pix, taking into account logarithmic axes */
+double pixwave(double pix, double crpix, double crval, double cdelt, bool logwave) {
+    if (logwave) {
+        return crval * exp((pix - crpix) * cdelt / crval); // following Greisen et al, 2006, EQ 4 & 5
+    } else {
+        return crval + (pix - crpix) * cdelt;
+    }
+}
+
+/* Calculate pixel position at spectral value val, taking into account logarithmic axes */
+int wavepix(double pix, double crpix, double crval, double cdelt, bool logwave) {
+    if (logwave) {
+        return (int)(crpix + crval / cdelt * log(pix / crval) + 0.5);
+    } else {
+        return (int)(crpix + (pix - crval) / cdelt + 0.5);
+    }
+}
+
 /*!
 Constructor: Allocates minimum space for a FITS conforming header (2880 bytes).
 The first line is set to "SIMPLE = T", the last to "END"
@@ -1155,7 +1465,7 @@ If the FITS key is a number (either floating point or integer),
 return the verbose copy of the key.
 */
 
-bool Fits::GetStringKey(const char *key, char *value) {
+bool Fits::GetStringKey(const char *key, char *value) const {
     char *pos, *npos, *lpos;
     int i, l = strlen(key);
     bool rv = FALSE;
@@ -1331,6 +1641,8 @@ double Fits::getCRVAL(int which) const {
 	return rv;
 }
 
+// Aug 27, 2020: Replaced by proper handling in readWCSinfo
+/*
 double Fits::getCDELT(int which) const {
 	double rv = 1.0;
 	char key[256];
@@ -1343,6 +1655,42 @@ double Fits::getCDELT(int which) const {
 	
 	return rv;
 }
+*/
+
+double Fits::getCDELT(int which) const {
+    double rv = 1.;
+
+    if (which < 3) {
+        wcsinfo info;
+        info = readWCSinfo(*this);
+        if (which == 1) rv = info.xinc;
+        else if (which == 2) rv = info.yinc;
+    } else {
+        char key[256];
+
+        sprintf(key, "CD%i_%i", which, which);
+        if (!GetFloatKey(key, &rv)) {
+            sprintf(key, "CDELT%i", which);
+            GetFloatKey(key, &rv);
+        }
+    }
+    return rv;
+}
+
+bool Fits::isAxisLog(int which) {
+    char key[9];
+    sprintf(key, "CTYPE%i", which);
+
+    char ctype[81];
+    bool logwave = false;
+
+    if (GetStringKey(key, ctype)) {
+        if (strlen(ctype) == 8) {
+            if (!strncmp((char *)(&ctype[4]), "-LOG", 4)) logwave = true;
+        }
+    }
+    return logwave;
+}
 
 /*!
 Read a FITS header from a file.
@@ -1621,7 +1969,7 @@ char * Fits::GetHeader() const {
        axis is given first, and to assume default values if any of the
        keywords are not present.
 */
-wcsinfo readWCSinfo(Fits &source) {
+wcsinfo readWCSinfo(const Fits &source) {
 	wcsinfo info;
 
     char ctype[81];
