@@ -90,6 +90,17 @@ int pixpos(double xpos, double ypos, double xref, double yref,
       double xrefpix, double yrefpix, double xinc, double yinc, double rot,
       char *type, double *xpix, double *ypix, int *status);
 
+int
+worldpix (double xpos, double ypos,
+          double xref, double yref,
+          double xrefpix, double yrefpix, double xinc, double yinc, double rot,
+          double cd1_1, double cd1_2, double cd2_1, double cd2_2,
+          char *type,
+          double *xpix, double *ypix);
+
+double pixwave(double pix, double crpix, double crval, double cdelt, bool logwave);
+int wavepix(double pix, double crpix, double crval, double cdelt, bool logwave);
+
 /*
  * Global variables
  */
@@ -226,6 +237,8 @@ public:
     int  CountTableColumns(const char *fname, const int extension);
     bool GetBintableColumn(int column, Fits &result);
     bool GetBintableColumn(int column, dpStringList &result);
+    bool GetAsciitableColumn(int column, Fits &result);
+    bool GetAsciitableColumn(int column, dpStringList &result);
     nodeEnum GetBintableType(int column);
 	bool WriteFITS(const char *fname);
     bool WriteFITSExtension(const char *fname);
@@ -256,7 +269,7 @@ public:
 	bool GetIntKeys(const char *, int *, int);
 	bool GetFloatKey(const char *, double *) const;
 	bool GetFloatKeys(const char *, double *, int);
-	bool GetStringKey(const char *, char *);
+    bool GetStringKey(const char *, char *) const;
 	bool GetStringKeys(const char *, char **, int);
 	bool tfieldWidth(int column, int *bytes, int *repeat, char *type);
 	bool DeleteKey(const char *);
@@ -327,6 +340,27 @@ public:
 		return 0.0;
 	}
 
+/*!Return value at specified sub-pixel position (2D-Case, Fortran-Notation).
+ * Uses linear interpolation.*/
+    inline double SubPixel(const double x1, const double x2) {
+        int j = (int)x1;
+        int k = (int)x2;
+/*
+        if (j < 1) j = 1;
+        else if (j >= Naxis(1)) j = Naxis(1) - 1;
+        if (k < 1) k = 1;
+        else if (k >= Naxis(2)) k = Naxis(2) - 1;
+*/
+        double fy0 = 0., fy1 = 0., fy2 = 0., fy3 = 0.;
+        if (j > 0 && j <= Naxis(1) && k > 0 && k <= Naxis(2)) fy0 = ValueAt(F_I(j, k));
+        if (j >= 0 && j < Naxis(1) && k > 0 && k <= Naxis(2)) fy1 = ValueAt(F_I(j+1, k));
+        if (j >= 0 && j < Naxis(1) && k >= 0 && k < Naxis(2)) fy2 = ValueAt(F_I(j+1, k+1));
+        if (j > 0 && j <= Naxis(1) && k >= 0 && k < Naxis(2)) fy3 = ValueAt(F_I(j, k+1));
+        double t = (x1 - j);
+        double u = (x2 - k);
+        return (1.0 - t) * (1.0 - u) * fy0 + t * (1.0 - u) * fy1 + t * u * fy2 + (1.0 - t) * u * fy3;
+    }
+
 /*!error message reporting*/
 	inline bool fits_error(char *msg) { dp_output("%s\n", msg); return FALSE; }
     inline bool reportError(dpint64 status) {
@@ -772,7 +806,8 @@ public:
 
 	double getCRPIX(int which) const;
 	double getCRVAL(int which) const;
-	double getCDELT(int which) const;
+    double getCDELT(int which) const;
+    bool isAxisLog(int which);
     double getBSCALE() { return bscale; }
     double getBZERO() { return bzero; }
 
@@ -871,6 +906,7 @@ int Boxcar(Fits & a, int size, int method = 0, int method2 = 0);
 int get_naxis3(char *fname);
 void getFitsFile(char *, Fits &);
 float *floatdata(const Fits &);
+double *doubledata(const Fits &);
 
 /*
  * Fits cube handling
@@ -906,7 +942,7 @@ void dp_stat(Fits &v, double *min, double *max, double *flux, double *avg, bool
 /*
  * WCS information
  */
-wcsinfo readWCSinfo(Fits &);
+wcsinfo readWCSinfo(const Fits &);
 
 void voronoi(Fits &, Fits &, Fits &, double, Fits &, int);
 
