@@ -265,8 +265,10 @@ bool cube_avg(Fits & cube, Fits & result, int first, int last)
     double value;
 	
 	if (cube.Naxis(0) != 3) {
-		dp_output("cube_avg: input image is no cube.\n");
-		return FALSE;
+        result.copy(cube);
+        return TRUE;
+//		dp_output("cube_avg: input image is no cube.\n");
+//		return FALSE;
 	}
 
     if (!result.create(cube.Naxis(1), cube.Naxis(2))) return FALSE;
@@ -316,8 +318,10 @@ bool cube_avg(Fits & cube, Fits & result, double reject)
 	double value;
 	
 	if (cube.Naxis(0) != 3) {
-		dp_output("cube_avg: input image is no cube.\n");
-		return FALSE;
+        result.copy(cube);
+        return TRUE;
+//		dp_output("cube_avg: input image is no cube.\n");
+//		return FALSE;
 	}
 	if (!result.create(cube.Naxis(1), cube.Naxis(2))) return FALSE;
 	result.CopyHeader(cube);
@@ -360,8 +364,10 @@ bool cube_avg(Fits & cube, Fits & result, int first, int last, double reject)
 	double value;
 	
 	if (cube.Naxis(0) != 3) {
-		dp_output("cube_avg: input image is no cube.\n");
-		return FALSE;
+        result.copy(cube);
+        return TRUE;
+//		dp_output("cube_avg: input image is no cube.\n");
+//		return FALSE;
 	}
 	if (!result.create(cube.Naxis(1), cube.Naxis(2))) return FALSE;
 	result.CopyHeader(cube);
