@@ -507,6 +507,20 @@ bool Fits::tfieldWidth(int column, int *bytes, int *repeat, char *type) {
 	if (!GetStringKey(key, tform)) return FALSE;
 	*repeat = 1;
 	c = 0;
+
+// ASCII Table
+    if (extensionType == TABLE) {
+        c = 1;
+        *bytes = tform[c] - '0';
+        c++;
+        while (c < strlen(tform) && isdigit(tform[c])) {
+            *bytes = 10 * (*bytes) + (tform[c] - '0');
+            c++;
+        }
+        *type = tform[0];
+        return TRUE;
+    }
+
 	if (isdigit(tform[c])) {
 		*repeat = tform[c] - '0';
 		c++;
@@ -716,6 +730,155 @@ bool Fits::GetBintableColumn(int column, dpStringList &result) {
     return retValue;
 }
 
+/*
+read the nth FITS Asciitable column and return dpStringList
+*/
+bool Fits::GetAsciitableColumn(int column, Fits &result) {
+    bool retValue = false;
+    int tfields;
+    if ((column < 1) ||
+        (!GetIntKey("TFIELDS", &tfields)) ||
+        (tfields < column))
+    {
+        return false;
+    }
+
+    // in ASCII Tables, the first byte is specified by the TBCOLn keyword
+    char key[9];
+    int bytes  = 0,
+        width  = 0,
+        repeat = 0;
+    char fieldtype;
+
+    sprintf(key, "TBCOL%i", column);
+    if (!GetIntKey(key, &bytes)) return false;
+    bytes--;
+
+/*
+    // go through the TFORMi keywords and count the bytes
+    for (int i = 1; i < column; i++) {
+        if (!tfieldWidth(i, &width, &repeat, &fieldtype)) {
+            return false;
+        }
+        bytes += repeat * width;
+    }
+*/
+    if (!tfieldWidth(column, &width, &repeat, &fieldtype)) {
+        return false;
+    }
+    width *= repeat;
+
+    FitsBitpix bits = NA;
+    switch (fieldtype) {
+        case 'I': bits = I4; break;
+        case 'F':
+        case 'E': bits = R4; break;
+        case 'D': bits = R8; break;
+        default: bits = NA; break;
+    }
+    if (bits == NA) {
+        return false;
+    }
+
+    result.create(Naxis(2), 1, bits); // ASCII Tables don't have a repeat field!
+
+    for (int y = 0; y < Naxis(2); y++) {
+        dpString t;
+        for (int x = bytes; x < bytes + width; x++) {
+            t.append(1, (char)i1data[C_I(x, y)]);
+        }
+        t.replace("D", "e");
+        t.replace("d", "e");
+        t.replace("E", "e");
+        switch (bits) {
+            case I4:
+                result.i4data[y] = std::stol(t); break;
+            case R4:
+                result.r4data[y] = std::stof(t); break;
+            case R8:
+                result.r8data[y] = std::stod(t); break;
+            default: break;
+        }
+    }
+    retValue = true;
+
+    double value;
+    sprintf(key, "TSCAL%i", column);
+    if (GetFloatKey(key, &value)) {
+        result.bscale = value;
+    }
+    sprintf(key, "TZERO%i", column);
+    if (GetFloatKey(key, &value)) {
+        result.bzero = value;
+    }
+
+    // get column name
+    sprintf(key, "TTYPE%i", column);
+    GetStringKey(key, result.columnName);
+    result.extensionType = BINTABLE;
+
+    return retValue;
+}
+
+/*
+read the nth FITS Asciitable column and return dpStringList
+*/
+bool Fits::GetAsciitableColumn(int column, dpStringList &result) {
+    bool retValue = false;
+    int tfields;
+    if ((column < 1) ||
+        (!GetIntKey("TFIELDS", &tfields)) ||
+        (tfields < column))
+    {
+        return false;
+    }
+
+    int bytes  = 0,
+        width  = 0,
+        repeat = 0;
+    char fieldtype;
+    char key[9],
+         value[72];
+
+    sprintf(key, "TBCOL%i", column);
+    if (!GetIntKey(key, &bytes)) return false;
+    bytes--;
+
+/*
+    // go through the TFORMi keywords and count the bytes
+    for (int i = 1; i < column; i++) {
+        if (!tfieldWidth(i, &width, &repeat, &fieldtype)) {
+            return false;
+        }
+        bytes += repeat * width;
+    }
+*/
+
+    if (!tfieldWidth(column, &width, &repeat, &fieldtype)) {
+        return false;
+    }
+    width *= repeat;
+
+//    if (fieldtype == 'A') {
+        for (int y = 0; y < Naxis(2); y++) {
+            dpString t;
+            for (int x = bytes; x < bytes + width; x++) {
+                t.append(1, (char)i1data[C_I(x, y)]);
+            }
+            result.push_back(t);
+        }
+        retValue = true;
+//    }
+
+    // get column name
+    sprintf(key, "TTYPE%i", column);
+
+    GetStringKey(key, value);
+    result.setColumnName(value);
+
+    return retValue;
+}
+
 nodeEnum Fits::GetBintableType(int column) {
     int tfields;
     if ((column < 1) ||
@@ -752,6 +915,7 @@ nodeEnum Fits::GetBintableType(int column) {
         case 'J':
         case 'K':
         case 'E':
+        case 'F':
         case 'D':
         case 'C':
         case 'M':
